                             esl functions        first version 17.10.1999
                             *************        revised       05.11.1999 
General                                           revised       08.01.2000
*******                                           revised       09.04.2000

This is a description of all "built in" esl functions. The functions are
sorted alphabetically.
The format of a function entry is

==============================================================================
   atan(a1 [,a2])
   example(s):     atan(1)      atan(3,2)
   Mnemonic (only in a few cases)
   corresponding scheme function
   General description, description of the arguments, description of return
   value, evaluation of arguments (if not standard), actions (if any). Hints
   on error or invalid arguments.
   see also: (References to related functions, in some cases)
==============================================================================

Optional arguments are included in square brackets ([ ... ]).
The description of the arguments and/or return values will be omitted, if
this is obvious from general description. The corresponding scheme function
is mentioned if there is one, even if it does not correspond completely to the
esl function. So, if you know scheme, do not assume the "corresponding scheme
function" to be completely semantically equivalent.
Different names for the arguments are used. Usually, x, y, ... are numbers
(float, integer and, possibly, rational are allowed), n, m, ... are integers,
s, t, ... are strings, l, l1, l2, ... are lists.
If no evaluation scheme is mentioned, then standard evaluation will take
place. This means that first all arguments are evaluated, then the values of
the arguments are passed to the function. If you want to pass a symbol in an
argument position where standard evaluation takes place, you have to quote
that symbol ('sym instead of sym).
Examples are not given for all functions. In some cases, an example consists
of several statements.
The notation
  fun(arg1,arg2, ...)  ==> expr
should be read as "evaluates to" or "returns". expr is the value of the
function invokation.

Note 1: 
The output of esl is still "scheme output", e.g. lists a written in the form
(a1 a2 a3 ...). If in the examples for an esl function the output is not a
simple thing like a number or a string, the output is given in a form more
similar to esl input, e.g., a list in the form {a1, a2, ...}.
Another problem arises in connection with symbols. All esl symbols that are
not "built in" are prepended by a dollar sign before they are processed by the
scheme system. Because esl output is essentially scheme output, the output
of, say 
'sym
is
$sym
In most cases, this does not matter, because the prepending of the dollar
sign is performed by the parser in a consistent way. However, when in the
examples of esl output below a symbol occurs, it will be shown without the
dollar sign.
The dollar signs are prepended to avoid confusion with internal symbols
(predefined by scheme, or defined by esl).

Note 2:
Although it is surely not desirable, the semantics of esl expressions might
depend in some cases on the underlying scheme systems in a non-obvious way.
The most prominent example in this respect are the rational numbers (something
like 7/13 is elementary data in some scheme implementations, in other not).
The problem is usually not present if these are intermediate results. However,
if you output numbers that are meant to be input for other programs, a problem
might arise. Simple output operations might output something like 7/13, which
will usually not be recognized as a valid number by other programs.
Another problem is the order of argument evaluation. In scheme, evaluation
order is only guranteed for some syntactical constructs, not for usual 
procedures. So, do not write code that depends on the evaluations order of
function arguments, unless you are willing to accept scheme system dependence.
Another system dependency is connected with complex numbers.

Note 3:
There is a "meta rule" for the names of predicate functions. In Scheme, these
functions usually have a question mark (?) as last character. Because question
marks are not allowed in esl names, they are usually replaced by the letter q.

****************************************************************************

abs(x)
example: abs(-3.5)                  ==> 3.5
scheme function: abs
Returns the absolute value of its argument. The argument must be a number.


append(l1 [,l2, ...])
example: append({2,3,5},{'a,'b,'c,'d},{-4,3+6})  ==> {2,3,5,a,b,c,d,-4,9}.
scheme function: append
Returns the list that is the concatenation of the arguments. All arguments
should be lists.


arraydim(a)
example: a :: [1..5,0..3],
         arraydim(a)                ==> 2
Returns the number of dimensions of its arguments, which should be an array.
(see also: lowindex, highindex)


arrayq(a)
Returns #t (true) if the argument is an array, otherwise, returns #f (false).


atan(y [,x])
examples:     atan(3),     atan(3,2)
scheme function: atan
Returns the arcustangens of its argument (in radian) when invoked with one
argument y (y must be a number). You will always have -pi/2 < atan(y) < pi/2. 
If invoked in the form atan(y,x), then it returns the angle phi between 0 and
2*pi for which x = cos(phi) and y = sin(phi) holds. This is useful for the
inversion of the transformation (r,phi) -> (r*cos(phi),r*sin(phi)). Both 
arguments must be numbers.


ceiling(x)
examples:    ceiling(3.1)      ==> 4.0
             ceiling(-3.1)     ==> -3.0
scheme function: ceiling
Returns the "ceiling" of the argument (a number). ceiling(x) is the least
integer greater than or equal to x. The type of the result is float. Use
inexact2exact to obtain an integer type: inexact2exact(ceiling(3.1)) ==> 4
(see also: floor, truncate, inexact2exact, round)


char(s)
example:   char("Kuh")         ==> #\K     (the character K)
The argument must be a nonempty string. char returns the first character of
the string. The result has character data type. This is different from a
string that consists of only one character.
Note: There is no syntax for character constants in esl yet. So, if you
really need character type data, use the char function.


charinstring(s,t)
example:  charinstring("home","Der Ochse")  ==> (#\h #\s #\e) (see below)
Checks if the first character from the string s does occur in the string t.
If this is the case, returns the list whose elements are the characters of
the substring of t that begins with the first occurence of the first character
of s. Otherwise, returns #f (false).
Both arguments must be strings, s must be nonempty. All characters of s but
the first are ignored.
A typical easy application: s consists of exactly one character. Then
charinstring returns a true value if that character occurs in t, otherwise #f.
The format of the output of the example given should indicate that the result
is a list of characters. It will not be displayed this way. What you will see
is        (h s e)


charq(c)
example: charq(char("A"))            ==> #t
Returns #t if c is a character (has data type character), #f otherwise.


closeout(port)
scheme function: close-output-port
closeout is called for action. It closes an output port, typically opened
previously with openoutfile.
(see also: openoutfile, display, displayln)


concatstringlist(sl [,delim])
example: concatstringlist({"Das","neue","Haus"})       ==> "DasneueHaus"
         concatstringlist({"Das","neue","Haus"},"_:_") ==> "Das_:_neue_:_Haus"
Returns the concatenation of the elements of the sl (sl must be a list of
strings). If delim (also a string) is specified, then delim is inserted
between every two strings of sl.
Note: For simple string concatenation, you can also use the operator "//"
(two slashes).


cons(f,r)
Examples : cons("AB",5)       ==>    "AB" . 5
           cons(5,{4,3,2,1})  ==>   {5,4,3,2,1}
Scheme functions: cons
Returns the pair composed of its arguments; f becomes the "first" (car) of
the pair, r becomes the "rest" (cdr) of the pair.


cos(a)
Example: cos(pi/4)            ==>   0.70710678 ... 
Scheme function: cos
Returns the cosine of its argument. The argument must be an angle measured in
radians (not in degrees).


cosd(ad)
Example: cosd(25)             ==>   0.906307787 ...  
Returns the cosine of its argument. The argument must be an angle measured in
degrees (not in radians).


ddlistq(l)
Example: ddlistq(44..55)    ==> #t
         ddlistq({44,55})   ==> #f 
Returns #t (true) if the argument is a list whose first element is the symbol
dd_ (The letter d, the letter d, the underscore), otherwise, returns #f.
Lists starting with the symbol dd_ are returned by the .. (dot, dot)-operator.


display(a1 [,a2, ...])
display(p,a1 [, a2, ...])
Examples: display(5,a,5*9)
          display(out,5,b,{4,5,c})
Scheme function: See below.
Takes any number of arguments. If the first argument is an output port, then
the following arguments are written to that port, otherwise all arguments are
written to the standard output. The output is written in the sense of the 
scheme function "display", i.e. without quote arround strings, without masking
special characters.
display is usually invoked for action, it returns the empty list.
(see also: displayln, write, writeln)


displayln(a1 [,a2, ...])
displayln(p,a1 [, a2, ...])
Examples: displayln(5,a,5*9)
          displayln(out,5,b,{4,5,c})
See "display". The only difference to "display" is that a newline character
is written at the end of the output. 
(see also: display, write, writeln)


displaysl(sl)
Example: displaysl({"This","is a","list","of","strings."});
Mnemonic: display string list.
The arguments should be a list of strings. This list is written to the
standard output. A newline is inserted at the end of every string.
displaysl returns the empty list. It is called for action.


dropdd(l)
Examples: dropdd(44..55)  ==> {44,55}    (the list containing 44 and 55)
          dropdd({44,string2symbol("dd_"),11..12})  ==> {44,{11,12}}
Removes recursively all occurences of the symbol dd_ from the list l.


dropmulti(l)
Example: dropmulti({22,14,14,14,{12,"a","a",0}})  ==> {22,14,{12,"a","a",0}})
Removes from equal consecutive elements in the list l all but one. Does
not process elements of the list that are lists themselves in the same way,
i.e. dropmulti does not work recursively. Equality is checked in the sense of
the scheme procedure "equal?". Roughly speaking, this means that elements
that appear equal are considered equal.


esl2scheme(s)
esl2scheme(l)
Examples: esl2scheme("plotwords")  
          esl2scheme({"plot","plotwords"})
Translates one or more file(s) containing esl statements to one scheme file.
The argument must be a string denoting a file with esl code, or a list of
such strings. If only a simple string (without extension, path) is specified,
the file name completion logic is the same a with the load function.
(see also: loadscheme)



evalstring(s)
Example:  evalstring("a::6,f(x)::x*x+7;f(a)")   ==> 43
Evaluates the string s as esl statements in the top level environment. The
content of the string s should be an esl statement or a sequence of esl
statements, separated by comma or semicolon.
evalstring does essentially what would have been done if you had entered the
string interactively from the esl prompt. evalstring writes the result of all
top level evaluations to standard output. It returns the result of the last
evaluation unit within the string. The "last evaluation unit" is determined in
the following way: If there is now semicolon in the string, it is the whole
string content. Otherwise, it is the part of the string after the last
semicolon, if this part is nonempty. If a semicolon is the last character of
the string, it is treated as if this semicolon was not present. If after the
last semicolon there are one or more whitespace characters, an empty list is
returned.
Typically, evalstring is invoked for action, so do not bother too much.
The string s may even contain a call to evalstring, e.g. something like that:
evalstring("a::5,evalstring(\"a++\"),displayln(a);a+2")
This will display 6 and return 8.
You may compare evalstring to "load". load evaluates esl code contained in a
file, evalstring evaluates esl code contained in a string.
(see also: load)


ew()
Example: ew()
ew invokes a text editor. You may create or edit any text, typically you
will edit esl code. When you save and exit the editor, the text will be
evaluated. When you invoke ew again, the editor starts with the last version
of the text. So you may remove errors you possible made in the previous editor
session. When you cancel the editor, the last saved version of the text will
be executed (if any). 
ew needs to know three things: The name of the editor program, the name of a
temporary file where it stores the current content, and the way to call an
external program. The name of the temporary file is contained in the variable
$workfile. The default name is ".esl_work_file.e", you may change it using
something like
$workfile : "my_new_work_file_name";
The directory is the current directory. Of course, you may change this if
you specify a complete path.
The specification of the editor and the way to invoke the editor from within
esl depends on the scheme system. It may even be impossible to invoke an
editor from esl, this means that ew() will not work for these scheme systems.


exp(x)
Example: exp(2)            ==> 7.389056 ... (= e*e)
Scheme function: exp
Returns the exponential function of its argument, which should be a number.


first(l)
Example: first({3+2,6})    ==> 5
Scheme function: car
Returns the "first" (the "car") of its argument, which should be a pair.
(see also: second, third, fourth, firstoffirst, listref, rest, restofrest)


firstoffirst(l)
Example: first({{4,5},7})  ==> 4
Scheme function: caar
Returns the first of the first of its argument, which should be a pair whose
"first" should be a pair.
(see also: first, second, third, fourth, rest, restofrest)


floor(x)
Examples:  floor(3.4)      ==>  3.0
           floor(-3.4)     ==>  -4.0
Scheme function: floor
Returns the "floor" of its argument, which should be a number. The floor of x
is the greatest integer that is not greater then x. However, the result has
float type. If you want integer type, you should use inexact2exact in the
following way: inexact2exact(floor(3.4))  ==> 3
(see also: ceiling, truncate, inexact2exact, round)


fourth(l)
Example: fourth({'a,'b,'c,'d,'e})   ==> d  (the symbol d)
Scheme function: cadddr
Returns the fourth element of the list l. This element should be present.
(see also: first, second, third, firstoffirst, listref, rest, restofrest)


highindex(a,d)
Example:  a :: [2..6,1..9],
          highindex(a,2)  ==>  9
Returns the highest index for dimension d of array a. Therefore, a should be
an array, d should be an integer between 1 and the number of dimensions of a.
(see also: arraydim, lowindex)


imag(z)
Example: imag(4+5i)            ==> 5.0
Scheme function: imag-part
Returns the imaginary part of its argument which should be a number. This
function will only work if the scheme system supports complex numbers.
(see also: real)
Note: There are some problems and system dependencies connected with complex
numbers.


index1dim(a,i1 [,i2, ...])
Example: 

a :: makematrix({{1,2,3,4,5,6},{11,12,13,14,15,16},{21,22,23,24,25,26}}),
i :: index1dim(a,2,4),
displayln("one-dimensional index is",i),              *** displays 9
displayln(a[2,4]),                                    *** displays 14
displayln(vectorref(second(a),i));                    *** displays 14
vectorset(second(a),i,1000);
displayln(vectorref(second(a),i));                    *** displays 1000

The first argument is an esl array, then there should follow as many index
values as there are dimensions of a (for an n-dimensional array a, there
should be n+1 arguments). The indices should be in the valid range.
This means, a[i1,i2 ...] should be a valid index expression. The function
index1dim returns the nonnegative integer that references the same element
as the index expression [i1,i2, ...] when a is considered a one dimensional
array, i.e. a vector in scheme sense. Note that the one-dimensional scheme 
array is the "second" (the "cadr") of the esl array. You can use vectorref and
vectorset to access the elements of a one dimensional array, see example
above. The advantage of the access via vectorref should be greater
"efficiency". There is no syntactic or semantic advantage.
(see also: vectorref, vectorset)


inexact2exact(x)
Example: inexact2exact(4.0)              ==> 4
Scheme function: inexact->exact
This should work like the corresponding scheme function. You may use it to
convert floating point numbers ("inexact numbers") to integers, or (if
supported by scheme system) rational numbers.
(see also: ceiling, floor, round, truncate)
Note: The dependence on the support of rational numbers in the scheme system
may lead to problems. inexact2exact(0.25) may evaluate to 1/4 or to 0!


integerq(r)
Example: integerq(-5)                    ==> #t
Scheme function: integer?
Returns true if the argument r is an integer, otherwise returns #f.


length(l)
Example: length({4,5,99})               ==> 3
Scheme function: length
Returns the length of the list l. It is an error if l is not a list.


list(a1, [,a2,...])
Example: list(1,2,3+2,4,"fife")           ==> {1,2,5,4,"fife"}
Scheme function: list
Returns the list composed of its (evaluated) arguments. 


list2string(l)
Example: list2string({char "a",char "b"})   ==> "ab"
Scheme function: list->string
Returns the strings composed of the elements of l. The elements of l should be
characters. 


listq(l)
Examples: listq({1,2,3,"oiuz",5})         ==> #t
          listq({})                       ==> #t
          listq(5)                        ==> #f
Scheme function: list?
Returns true if the argument is a list (even if it is the empty list), returns
#f otherwise.


listref(l,i)
Example: listref({7,4,1},2)               ==> 1
Scheme function: list-ref
Returns the i+1 th element of l. l is a list, i is a nonnegative integer. If
you started counting with 0, you would probably say that listref(l,i) returned
the i-th element of l. Compare this to the functions first, second, third,
fourth. second(l) returns the second element of l, while listref(l,2) returns
the third element of l.
Note: listref could have been defined such that listref(l,i) would return
the i-th element. However, listref works like the scheme procedure list-ref.
I do not know what is better in this place. However, I really know that I do
not like counting from zero. Saving an operator in some index calculations
or "being closer to internal memory management" cannot compensate for the
obfuscation introduced by counting from zero.
(see also: first, second, third, fourth)


ln(x)
Example: ln(2)            ==>  0.693147180559945
Scheme function: log
Returns the natural logarithm of its argument, which should be a nonnegative
number.
(see also: log)


load(filename)
loadsys(filename)

Example: load("esl_test")

Read a file that is supposed to contain esl code. That code is evaluated
(executed) in the top level domain. It is treated as if it was typed in in
interactive mode. The argument must be a string containing the name of
the file to be loaded.
When no path is specified, load will look for the file in the current
directory, loadsys will look for it in the system directory (/usr/local/esl
by default).
If you do not specify a file name extension, then ".e" (dot and the letter e)
is appended to the filename.
The content of filename may contain load invokations. This means, load can
be used recursively.
load is usually invoked for action (e.g., function definitions are loaded).
(see also: loadscheme)


loadscheme(filename)
Example: loadscheme("plot")
Loads a file containing scheme code. If you want to use this scheme code in
esl, then please note that esl can access only symbols defined in that file
if they have a dollar sign as first character. Do not type this dollar sign in
esl statements, because the parser adds the dollar automatically. Probably
you want to use scheme code generated from esl code with the esl2scheme
function. In this case the dollar signs have been added by esl2scheme, you
can use the symbols as if they had no dollar sign. Loading scheme code is
usually much faster than loading esl code.
Note: Possibly, there are still some inconsistency concerning search pathes
and file name completion logic.
(see also: load, esl2scheme)


log(b,x)
Example: log(3,81)              ==>  4.0            
Both arguments should be positive numbers. log(b,x) returns the logarithm to
base b of x.
(see also: ln)


lowindex(a,d)
Example:  a :: [2..6,1..9],
          lowindex(a,1)  ==>  2
Returns the lowest index for dimension d of array a. Therefore, a should be
an array, d should be an integer between 1 and the number of dimensions of a.
(see also: arraydim, highindex)


ls([a1, ...])
Example: ls()            the system command "ls"
         ls("-la")       the system command "ls -la" 
         ls("-a p*")     the system command "ls -la p*" 
         ls("-a","p*")   the same as the last example.
This implements the system command "ls" as esl function. Without arguments, it
means simply "ls". Any sequence of options and parameters to ls may be
specified as on or more string arguments. As space will be inserted between
"ls" and the first argument and between the arguments.
ls is invoked for action.
The esl function ls will only work if the system interface of the scheme
system is functional.


map(f,l)
Example: map(sqrt,{1,4,25,70+30})   ==> {1.0,2.0,5.0,10.0}
Scheme function: map
The first argument must be a function, the second argument must be a list.
The function f is applied to the elements of l, the results are returned as a
list.
Note: Neither the order of evaluation of the elements of l nor the order of
application of f to the elements of l are guranteed.


mathoutoff()
Turns off the "mathematical" form of output. See mathouton(),
(see also: mathouton)


mathouton()
Turns on the "mathematical" form of output for evaluated scheme expressions.
Normally, results, i.e. the values of the input esl expression, are displayed
in scheme form. The mathematical form tries to be closer to usual infix
notation. This feature is still incomplete, consider the present form
"experimental".
Of course, the output form does not matter for atomic results (numbers,
strings, symbols). Mathematical output should be in particular convenient
for the symbolic manipulation of mathematical functions.
Note: Only the normal output is affected. Expressions output explicitely with
the display function (or similar functions), as well as the output of the
parsing process (as turned on by parseon();) are not effected.
(see also: mathoutoff)

     
max(x1 [,x2, ...])
Example: max(2,sqrt(2),9,sqrt(256),4)   ==> 16.0
Scheme function: max
Returns the maximum of the values of its arguments. All arguments should be
numbers. Complex numbers are not allowed.
(see also: min)


min(x1 [,x2, ...])
Example: min(2,sqrt(2),9,sqrt(256),4)   ==> 1.414 ...
Scheme function: min
Returns the minimum of the values of its arguments. All arguments should be
numbers. Complex numbers are not allowed.
(see also: max)


nullq(l)
Examples: nullq({4,5})            ==>    #f
          nullq({})               ==>    #t
Scheme function: null?
Returns #t (true) if the argument is an empty list, otherwise returns #f.


number2string(x)
Examples: number2string(3.56)        ==> "3.56"
Scheme function: number->string
Converts its argument, which must be a number, to a string.


numberq(x)
Examples:      numberq(4.7)          ==> #t
               numberq("55")         ==> #f
Scheme functions: number?
Returns #t (true) if the argument is a number, otherwise returns #f.


openoutfile(s)
Examples: openoutfile("the_file_name")  ==> [output port for "the_file_name"]
Scheme function: open-output-file
Opens the file with name s (which must be a string) for output. The output 
port (to be used, e.g. in display(out, ...)) is returned. The filename may be
specified with a path. If the path is omitted, the file will be created in the
current directory.
(see also: closeout)


output( [s] )
Example: output("output_log")
         output()
Directs standard output to the file whose name is specified in the string s.
If the argument s is not specified, the output is redirected to the terminal.
Before directing the output to a new file or to terminal, the current output
file (if any) is closed. The path may be specified in the pathname; if not,
the output file is created in the current directory.


pairq(p)
Examples: pairq(cons(4,5))        ==> #t
          pairq(88)               ==> #f
Scheme function: pair?
Returns #t (true) if the argument is a pair, otherwise returns #f.


parseoff()
Turns off the display of the parsed (i.e. transformed to scheme) scheme
expression. parseoff sets the variable $showparse to #f (false). 
parseoff is called for action.
(see also: parseon)


parseon()
Turns on the display of the parsed (i.e. transformed to scheme) scheme
expression. parseon sets the variable $showparse to #t (true). This means
that the parsed esl expressions are written to standard output. You should
know scheme (at least a little) to check the parse result. Note that the
parser prepends all user created symbols by a dollar sign.
parseon is called for action.
(see also: parseoff)


pop(l)
Example: l::{5,3,2}, pop(l)  ==>  5
                     l       ==>  {3,2}
The argument l must be a variable whose value is a list. pop(l) returns
the first element of the list (i.e. first(l)), and sets the value of l to
the rest of the original value (i.e. with the return value removed). This
means, pop does not evaluate its argument in the standard way. pop will return
#f (false), if l is the empty list. It is an error if l is not a list. pop is
used for action and for the return value.
Together with push, pop can be used to manage a stack ("first in, first out").
(see also: push)  


procedureq(p)
Examples:         procedureq(sin)       ==>  #t
                  procedureq(x->x*x+2)  ==>  #t
                  procedureq("x*x")     ==>  #f
Scheme function:  procedure?
Returns #t (true) if the argument p is a procedure, otherwise returns #f.


push(a,l)
Example: l::{3,2}, push(7,l) ==> #<unspecified>
                          l  ==> {7,3,2}
The first argument is any valid expression, it is evaluated in the standard
way. The second argument must be a variable whose value is a list. push sets
the value of l to cons(a,l), i.e. the value of a is prepended to the list l.
This means, push does not evaluate its second argument in the standard way.
push is used for action.
Together with pop, push can be used to manage a stack ("first in, first out").
(see also: pop)


quicksort(l [,compproc])
Example:  quicksort({3,45,23,34,2,46,3},(x,y)->x>y)  ==> {46,45,34,23,3,3,2}
Returns l sorted. The argument l must be a list. commproc is a comparison
procdure, it takes two arguments and returns #t or #f. If compproc is omitted,
< (less than) is assumed. I think quicksort does not change its argument (i.e.
quicksort ist non-destructive).


real(z)
Example: real(4.3+5.2i)            ==> 4.3
Scheme function: real-part
Returns the real part of its argument which should be a number. This
function will only work if the scheme system supports complex numbers.
(see also: imag)
Note: There are some problems and system dependencies connected with complex
numbers.


rest(l)
Example: rest({3,2,45,22})        ==> {2,45,22}
Scheme function: cdr
Returns the "rest" (the "cdr") of its argument, which should be a pair.
(see also: first, second, third, fourth, firstoffirst, listref, restofrest)


restofrest(l)
Example: restofrest({3,2,45,22})        ==> {45,22}
Scheme function: cddr
Returns the rest of the rest of its argument, which should be a pair whose
rest is a pair.
(see also: first, second, third, fourth, firstoffirst, listref, rest)


reverse(l)
Example: reverse({3,2,45,22})           ==> {22,45,2,3}
Scheme function: reverse
Returns the reverse (list in opposite order) of its argument which should be
a list. Sublists are not reverted.


round(x)
Examples:        round(4.3)      ==> 4.0
                 round(4)        ==> 4.0
                 round(-4.3)     ==> -4.0
                 round(4.5)      ==> 4.0
                 round(5.5)      ==> 6.0
Scheme function: round
Returns the integer that is closest to ist argument. The argument must be a
number. The result type is float. Numbers of the form x.5 are rounded to the
nearest even number.
(see also: ceiling, floor, truncate)


roundn(x,n)
Example:        roundn(pi,4)     ==>  3.1416
Rounds x, which should be a number, to n digits after decimal point.
(see also: round)
Note: There is still a problem connected with roundn when the scheme system
supports rational numbers. This should be fixed soon.


scheme()
Terminates esl, returns to the scheme prompt. scheme() is called for action.
(see also: scm)


schemesymbol(s)
Example: schemesymbol("cadar")               ==> [primitive function cadar]
         schemesymbol("cadar")({{4,5,6},4})  ==>     5
         schemesymbol("+")(4,7)              ==>    11
The argument s must be a string that is, when converted to a symbol, a valid
symbol in the underlying scheme system. schemesymbol returns the value of this
symbol. For example, if the value of this symbol is a scheme procedure, this
procedure is returned. If in this case you use "schemesymbol(s)" in function
position in an esl expression, this scheme procedure is applied, as shown by
the examples above, to the following argument(s).
Thus, schemesymbol enables the use of scheme procedures that have no direct
counterpart in esl scripts. Furthermore, schemesymbol may be used to pass
a function as arguments, when this function is implemented as operator in esl.
For example, you cannot write something like
fun(*,5)
because the parser always tries to interpret * as prefix or infix operator,
and this is syntactically not possible here. You cannot even use
fun('*,5)
or
fun(&'*,5),
since the parser would still try to find an operand for * (the quote operator
clearly influences evaluation, but not parsing). However,
fun(schemesymbol("*"),5)
might be ok, if fun expects a two-argument function as its first argument.
Of course, you might as well use
fun( (x,y)-> x*y, 5)
Note 1:
If you want to use schemesymbol(s) as function in esl, the scheme symbol
corresponding to the string s must not be a macro or special syntax. So,
schemesymbol("or")(4<6,4>99)
will probably not work.
Note 2:
The schemesymbol function enables direct access to symbols of the scheme
system. It is a bypass to the parser. So, the dollar sign is not prepended to
the symbol corresponding to the string s. You may easily destroy the running
scheme and esl system in a hardly predictable way.
Consider schemesymbol a low level feature. Do not use it, if you can avoid it.
Note 3 (if you know scheme):
The implementation of the schemesymbol function is very easy. Without argument
check and a possible second argument to the eval procedure, it is simply
(define (schemesymbol s) (eval (string->symbol s)))


scm()
This is the same as scheme().
(see also: scheme)


second(l)
Example:   second({3,5,2})      ==>   5
Scheme function: cadr
Returns the second element of its argument which should be a list that
contains at least two elements.
(see also: first, third, fourth, firstoffirst, rest, restofrest, listref)


show(sl1 [,sl2])
This is meant to be a means for quick online information. So,
show(infix)
should display information on all infix operators, and
show(first)
should display information on the function first. However, show does not work 
as intended yet. Show does not evaluate its arguments in the standard way.
Symbols (like infix) should be passed without quote. You may use strings,
however, e.g.
show("infix")


sin(a)
Example: sin(pi/6)            ==>   0.5 
Scheme function: sin
Returns the sine of its argument. The argument must be an angle measured in
radians (not in degrees).


sind(ad)
Example: sind(25)             ==>   0.42261826 ...
Returns the sine of its argument. The argument must be an angle measured in
degrees (not in radians).


splitfilename(s)
Examples: splitfilename("/home/my.dir/file.name.ext")
                              ==>   {"/home/my.dir/","file.name",".ext"}
The argument must be a string. This string is considered a filename, possibly
including a path and a file extension. This filename (i.e. the string s) is
split into three parts (substrings): directory path, filename without path
and without extension, extension. splitfilename returns a list of this three
parts, in that order. The directory path is the part from the beginning of s
up to and including the last slash (rightmost) (/), if there is any slash in
the string. If there is no slash in s, the directory path is the empty string.
If there is a period (.) in the string such that no slash is right to this
period, then the part right to the rightmost periods of this kind (including
this period) is the "extension". The rest of s is considered the filename.


sqrt(x)
Example: sqrt(20.25)          ==>   4.5
Scheme function: sqrt
Returns the square root of its argument. If the scheme system supports complex
numbers, the argument may be any number. Otherwise, it must be a nonnegative
number.


string2list(s)
Scheme function: string->list
Returns the list containing the characters of the string, in order. The list
elements have data type "character". The argument s must be a string.


string2number(s)
Examples: string2number(" 044.500")   ==>  44.5
          string2number("Ochse")      ==>  #f
Scheme function: string->number
The argument s must be a string. If s can be interpreted as a valid number,
this number will be returned. If s is a string, but does not contain a valid
number representation, #f (false) will be returned.


string2symbol(s)
Example: string2symbol("Ochse")      ==>  'Ochse    [The symbol Ochse]
Scheme function: string->symbol
The argument must be a string. This string is transformed to the corresponding
symbol, i.e. to the symbol that has the same sequence of characters.


stringgeq(s1,s2)
Example: stringgeq("Affe","Affe")     ==> #t
         stringgeq("Aff","Affe")      ==> #f
         stringgeq("Affe","Aber")     ==> #t
         stringgeq("affe","Af")       ==> #f
Scheme function: string>=?
The arguments must be strings. stringgeq returns #t if s1 is "greater than" or
equal to s2, otherwise returns #f. This means s1 comes after s2 in alphabetic
order. Upper case letter are alphabetically less then lower case letters.
(see also: stringgtq, qstringleq, stringltq)


stringgtq(s1,s2)
Examples: stringgtq("Affe","Affe")     ==> #f
          stringgtq("Aff","Affe")      ==> #f
          stringgtq("Affe","Aber")     ==> #t
          stringgtq("affe","Af")       ==> #f
Scheme function: string>?
The arguments must be strings. stringgtq returns #t if s1 is "greater than"
s2. See stringgeq for information on string order.
(see also: stringgeq, stringleq, stringltq)


stringlength(s)
Example: stringlength("Ochse")       ==> 5
Scheme function: string-length
Returns the length of its argument which must be a string.


stringleq(s1,s2)
Examples: stringleq("Affe","Affe")     ==> #t
          stringleq("Aff","Affe")      ==> #t
          stringleq("Affe","Aber")     ==> #f
          stringleq("affe","Af")       ==> #t
Scheme function: string<=?
The arguments must be strings. stringleq returns #t if s1 is "less than" or
equal to s2. See stringgeq for information on string order.
(see also: stringgeq, stringtq, stringltq)


stringltq(s1,s2)
Examples: stringltq("Affe","Affe")     ==> #f
          stringltq("Aff","Affe")      ==> #t
          stringltq("Affe","Aber")     ==> #f
          stringltq("affe","Af")       ==> #t
Scheme function: string<?
The arguments must be strings. stringltq returns #t if s1 is "less than" s2.
See stringgeq for information on string order.
(see also: stringgeq, stringtq, stringleq)


stringq(s)
Examples: stringq("Ochse")            ==> #t
          stringq(5)                  ==> #f        
Scheme function: string?
Returns #t if the argument is a string, otherwise returns #f.


substring(s,i1,i2)
Example: substring("Ochse",3,4)       ==> "hs"
Scheme function: substr
The argument s must be a string, i1 and i2 must be integers,
1 <= i1 <= i2 <= stringlength(s). substring returns the substring of s that
starts with the i1-th letter and ends with the i2-th letter. The first letter
in s has number 1 (not 0).


sumf(f,start,end [,step])
Examples: sumf(i->i*i,1,5)         ==>   55   ( = 1+4+9+16+25)  
          sumf(i->i*i,1,5,2)       ==>   35   ( = 1+9+25)
f is a function of a real argument, start, end and (if specified) step are
numbers. The function f is evaluated with arguments start, start+step,
start+step+step, ... as long as the argument thus obtained is not greater than
end. If step is not specified a step of 1 will be assumed. You may specify
end smaller than start, in this case step has to be negative. start, end and
step do not have to be integers. However, because of rounding errors, you may
not obtain what you expect. So, sumf(i->1,1,2,1/20) might evaluate to 20,
although 21 could be expected. You might want to increase end a little.
sumf(i->1,1,2.001,1/20) should evaluate to the correct result 21.
Note: Alternatively, you may use the control construct sum.


symbolq(s)
Examples:     symbolq('Ochse)    ==>  #t
              symbolq("Ochse")   ==>  #f
Scheme function: symbol?
Returns #t (true) if the argument is a symbol, otherwise returns #f.


system([syscom])
Examples:   system()             ==>    {}  (Enters a system shell)
            system("ps ax")      ==>    0   (displays the output of the
                                             shell command "ps ax")
system() enters a shell. You can enter any unix command. Leave the shell
and re-enter esl with
exit
In this case, system will return the empty list.
system(syscom) will execate the shell command syscom, which must be a string
containing a valid command. In this case, system will return the exit code
of the shell command.


tan(a)
Example: tan(pi/4)            ==>   1.0 
Scheme function: tan
Returns the tangens of its argument. The argument must be an angle measured in
radians (not in degrees).


tand(ad)
Example: tand(-45)            ==> -1.0
Returns the tangens of its argument. The argument must be an angle measured in
degrees (not in radians).


third(l)
Example: third({1,2,3,4})     ==>  3
Scheme function: caddr
Returns the third argument of its argument, which must be a list. Note that
the third element of l is considered to have number the 3, not 2 (i.e.
counting starts with 1).


truncate(x)
Examples:  truncate(3.4)      ==>  3.0
           truncate(-3.4)     ==>  -3.0
           truncate(4.9)      ==>  4.0
Scheme function: truncate
Truncates its argument, which should be a number. The result has the decimal
part of its argument replaced by 0. However, the result has real type. If you
want interger type, you should use inexact2exact in the following way:
inexact2exact(truncate(-3.4))  ==> -3
(see also: floor, ceiling, inexact2exact, round)


vectorref(v,i)
Example: See index1dim
Scheme function: vector-ref
Gets a value from a one dimensional, zero based array. See description of
index1dim.
(see also: index1dim, vectorset)


vectorset(v,i,val)
Example: See index1dim
Scheme function: vector-set!
Sets a value in a one dimensional, zero based array. See description of
index1dim.
(see also: index1dim, vectorref)


write(a1 [,a2, ...])
Examples: write(5,"a",a,5*9)
Scheme function: See below.
Takes any number of arguments. All arguments are written to the standard
output. The output is written in the sense of the scheme function "write",
i.e. with quote arround strings, special characters are masked with a
backslash.
write is invoked for action, it returns the empty list.
Note: write(p, a1, ...), i.e. write with a port as first argument, does not
work yet.
(see also: display, displayln, writeln)


writeln(a1 [,a2, ...])
Examples: writeln(5,"a",a,5*9)
See "write". The only difference to "write" is that a newline is written
at the end of the output. 
Note: writeln(p, a1, ...), i.e. writeln with a port as first argument, does
not work yet.
(see also: display, displayln, write)
