                             esl operators          first version 09.01.2000
                             *************                updated 14.04.2000           
General
*******

This is a description of all "built in" esl operators. With respect to syntax,
there are three kinds of operators: prefix, infix and postfix. Infix operators
are placed between two operands, prefix operators are placed before their
(only) operand, and postfix operators are placed after their (only) operand.

The format of an operator entry in the following description is this:

==============================================================================
   a1 iop a2   [a1 and a2 are the operands of the infix operator iop]
      or   
   preop a     [a is the operand of the prefix operator preop]
      or
   a postop    [a is the operand of the postfix operator postop]

   binding powers lbp rbp    [lbp and rbp are the left and right binding 
                              power of an infix operator, only one binding
                              power is indicated for pre- and postfix
                              operators]
   (flattable?)
   example(s)
   corresponding scheme function
   [or corresponding scheme special syntax]
   General description, description of the operands, description of return
   value, evaluation of arguments (if not standard), actions (if any). Hints
   on error or invalid arguments.
   see also: [References to related functions, in some cases]
==============================================================================

The description of the operand(s) and/or return values will be omitted, if
this is obvious from general description. The corresponding scheme function is
mentioned if there is one, even if it is not completely to the esl function.
Different names for the operands are used. Usually, x, y, ... are numbers
(float and integer are allowed), n, m, ... are integers, s, t, ... are
strings, l, l1, l2, ... are lists.

There are infix (operator between two operands), prefix and postfix (operator
before, respectively after one operand) operators. 
The binding powers are given for infix operators in the form
lbp rbp
(example: 80 81)
The example means that the left binding power is 80 and the right binding
power is 81. Pre- and postfix-operators only have one binding power, because
they have only one operand. See below, Note 2, for some remarks on the control
of parsing by the binding powers.

Flattability is only indicated if present. Currently, the following operators
are flattable: *, +, and, or, >, >=, <, <=, //.
The last in this list (//) means string concatenation. Only infix operators
may be flattable. Flattability means: Nested structures or substructures of
an expression, grouped from the left, and containing only one kind of
operator, are combined to only one function call. Example:
((4*6)*7)*8
is transformed to the inner (scheme) representation
(* 4 6 7 8)
Because of the grouping property of the parser in case of missing parentheses,
the expression
4*6*7*8
is transformed as well to
(* 4 6 7 8)
However, 4*(6*(7*8))
will not be flatted. Flattability should not have any semantic impact.

If no evaluation scheme is mentioned, then standard evaluation will take
place. This means that first the operands are evaluated, then the values of
the arguments are passed to the operator. If you want to pass a symbol in an
operand position where standard evaluation takes place, you have to quote
that symbol ('sym instead of sym).

Examples are not given for all operators. In some cases, an example consists
of several statements.
The notation
  a1 op a2  ==> expr
for an infix operator op should be read as "evaluates to" or "returns". expr
is the value of the operator, a1 and a2 are the operands, op is the operator.
This is modified in the obvious way for prefix and postfix operators.
If instead of "expr" in an example the string "#<unspecified>" is given, this
means that the operators returns an unspecified value. Never rely on an
"#<unspecified>" value. This "unspecified" value may depend on the scheme
system, for example "#<void>" is possible instead of "#<unspecified>".

Note 1: Semantically, there is no principal difference between operators and
functions. Operators may have one or two operands, functions may have any
number of arguments.
Both can evaluate arguments (operands, respectively) in standard or
nonstandard way, both may be called for result, or for action, or for both.
The primary difference is syntactical.

Note 2: The binding powers of the operators are used by the parser for the
proper grouping of the operators. Binding powers are integers in the range
from 1 to 254. The exact rules for this binding power based grouping will be
explained in detail elsewhere.
For short, binding powers control the "priority" of operators (e.g. rules of
the kind: "3*5+6 means (3*5)+6, not 3*(5+6)"). If two operators compete for
one operand (in situations like " ... op1 a1 op2 ..."), then the left operator
(op1) will win if its right binding power is greater than or equal to the left
binding power of the right operator (op2). The operands (a1 in this example)
may itself be subexpressions of any complexity.
Of course, you may always force the desired grouping using parentheses.

Note 3: If the display of parsed input is set (using parseon() ), then esl
will show the result of parsing an input before evaluation. This way you may
check if parsing is performed as desired. Please note, however, that the
parsed input expression is shown in scheme notation (fully parenthesized,
operators and functions in the first position). Please note also that esl
variables are displayed with a prepending dollar sign (for example $var
instead of var). See note 4.
Display of parsed input can be switched off with the statement
parseoff()

Note 4: The output of esl is still "scheme output", e.g. lists a written in
the form (a1 a2 a3 ...). If in the examples for an esl operator the output is
not a simple thing like a number or a string, the output is given in a form
more similar to esl input, e.g., a list in the form {a1, a2, ...}. Another
problem arises in connection with symbols. All esl symbols that are not
"built in" are prepended by a dollar sign before they are processed by the
scheme system. Because esl output is essentially scheme output, the output
of, say 
'sym
is
$sym
In most cases, this does not matter, because the prepending of the dollar
sign is performed by the parser in a consistent way. However, when in the
examples of esl output below a symbol occurs, it will be shown without the
dollar sign.
In the current esl version (version 0.05, pathlevel 13) an additional output
form is implemented, called "math output". It is switched on by
mathouton();
and switched off by
mathoutoff();
Please, consider this feature experimental. It is an attempt to make output
of scheme expression more look like mathematical expressions. There are still
obvious deficits in this feature. You might try to use "math output" if you
use the "symfuns" module. Note that symfuns is also considered experimental.
Note on Note 4: The whole esl is experimental. If a feature is called
"experimental", this really means a higher level of "experimental".

Note 5: In some cases, the same symbol may denote different operators. So,
+ and - may be the usual infix operators, or they may denote prefix operators.
These are considered different operators, hence there are two entries in
the following operator description list. The parser can tell what is meant by
the "syntactical environment". For example, in (-6*8) the minux sign cannot
denote an infix operator, because it is impossible that there is an opening
parenthisis left to an infix operator. Usually, you do not have to care about
this, since is complies with mathematical notation.

Note 6: The operators are denoted by single special characters (+, *, ...),
by combinations of special characters (++, --, <=, ...) or by "names"
in the usuall sense (div, mod, ...). If an operator is denoted by a name,
you will usually have to use spaces (whitespace) before and after this
operator name. Example:
x div y
instead of
xdivy
because the latter would mean the symbol xdivy. It would be possible to type
5div 2, but this is not recommended.
In some cases you can omit this whitespace without problems, even if the
operator is denoted by a name. Example:
5 div(6+7)
In most cases you can omit the whitespace around the operator if the operator
consist of special characters. So, of course you may type
5*7
or
6+8.7*9
In some cases a single special character may denote an operator and at the
same time this special symbol is part of the denotation of another operator:
< means "less than", <= mean "less than or equal", while "< =" (less sign,
space, equal sign) would mean "less than operator, followed by equal
operator". The latter does not make sense (it is a syntactical error). On the
other hand, there could be two valid interpretations of, say
--x
First, it is the predecrement-operator, applied to x, second, it could mean
-(-x)
(the prefix minus applied twice to x). esl always assumes the first
interpretation. If you really mean the second expression, type "- -x" or use
parentheses.
The general rule is: if two ore more consecutive special characters denote
an operator, then this operator is assumend, even if the special characters
for themselve (or shorter pieces of the sequence) do denote other operators.
Another example: -> (hyphen, greater sign) denotes the "lambda", it is used
to define a (usually unnamed) function. Of course, - alone is the minux
operator and > alone is the "greater than" operatoer. There should be no
problem of ambigouity here because the sequence "minus operator, followed by
greater than operator" does not make sense. However, the sequence "less than
operator, followed by minus operator" can make sense (like in "a <-5"), so if
there would be an operator denoted by "<-", then you would have to write the
sequence "less than operator, followed by minus operator" in the form "< -"
(space between < and -). Because of this problem I do not think it is a good
idea to denote an operator by "<-".
A short summary of this Note 6:
- spaces (whitespace) around operators can be omitted, if there is no danger
  of confusion.
- if in doubt, use whitespace.


Note 7: It is very easy to change the binding powers of the operators. It
is also easy to change other syntactic properties of operators (names of
operators, flattability). You only have to edit the esl source code and
make a simple change. However, one should be careful with these
possibilities. Files with esl code could become syntactically invalid.


There are several categories of operaters: arithmetic, comparison, logic,
other. The desription of the operators is ordered by these categories. There
is no special order within the catergories.
****************************************************************************

Arithmetic operators
********************
If not stated otherwise, operands to arithmetic operators are numbers (integer
or float). If the scheme systems used as base for esl supports complex
arithmetics, then complex operands will be accepted as well. Then, of course,
the result will usually have type complex. This is not mentioned in the
description of the operators.
Generally speaking, esl arithmetics is essentially scheme arithmetics. So,
the arithmetics may differ for different scheme bases of esl.


x * y
Binding powers: 100 100
flattable
Example:        5 * 6.5     ==> 32.5
Scheme function:  *
Returns the product of its operands, which should be numbers. The result
has type integer, if both operands are integers, else the result has type
float.


x / y
Binding powers: 100 100
Example:        13.5 / 4.5     ==> 3.0
Scheme function:  /
Returns the quotient of its operands, which should be numbers. Of course,
division by zero is not valid. The result has type integer, if both operands
are integers and x is an integer multiple of y. Otherwise, the result has
type float.


x + y
Binding powers: 90 90
flattable
Example:        44 + (-4.5)    ==> 39.5
Scheme function: +
Returns the sum of its operands. The result has type integer, if both operands
are integers, else the result has type float.


+ x
Binding power: 91
Example:       +x              ==> 7 (if the value of x is 7)
Essentially does not do anything, it returns the value of its operand. 
It might be viewed as the "identity operator". It will, however, onlye
work for operands that evaluate to numbers. The unary + is present for
symmetry with the unary -. Some automatic syntactical manipulations might be
easier with unary + present. In expressions of the form
+ n
where n is a number (not a variable whose value is a number) and + cannot
be viewed as infix operator, for example
+9
the "+" is consindered part of the number, not as operator. This is even
true if there is whitespace between the + sign and the rest of the number.
This latter fact should possibly be considered a bug.
(see also: - x) 


x - y
Binding powers: 90 90
Example:        44 - (-4.5)    ==> 48.5
Scheme function: -
Returns the difference of its operands. The result has type integer, if both
operands are integers, else the result has type float.


- x
Binding power: 91
Example:       -x              ==> 7 (if the value of x is 7)
Returns the negative of its operand. The result type ist the type of the
operand.
In expressions of the form
- n
where n is a number (not a variable whose value is a number) and - cannot
be viewed as infix operator, for example
-9
the "-" is consindered part of the number, not as operator. This is even
true if there is whitespace between the - sign and the rest of the number.
(see also: + x)


++ x
Binding power: 112
Example:  ++x                ==>  9  (if the value of x was 8 before "++x")
            x                ==>  9  (this is the value of x after "++x", if
                                      this value was 8 before).
This is the "prefix increment" operator, similar to the corresponding C
operator.
The operand must be variable whose value is a number. The operator returns
this number, incremented by one, and sets the variable to this incremented
value. This means, the operator does not evalute its operand in the standard
way. The prefix operator ++ is used for action and for return value, or only
for action.
Note: It might be desirable to allow expressions like
++a[5] or ++first(l), where a is an array and l is a list. This does not work
yet.
(see also: --x, x++, x--)


-- x
Binding power: 112
Example:  --x                ==>  7  (if the value of x was 8 before "--x")
            x                ==>  7  (this is the value of x after "--x", if
                                      this value was 8 before).
This is the "prefix decrement" operator, similar to the corresponding C
operator.
The operand must be variable whose value is a number. The operator returns
this number, decremented by one, and sets the variable to this decremented
value. This means, the operator does not evalute its operand in the standard
way. The prefix operator -- is used for action and for return value, or only
for action.
Note: It might be desirable to allow expressions like
--a[5] or --first(l), where a is an array and l is a list. This does not work
yet.
(see also: ++x, x++, x--)


x ++
Binding power: 111
Example:    x++              ==>  8  (if the value of x was 8 before "x++")
            x                ==>  9  (this is the value of x after "x++", if
                                      this value was 8 before).
This is the "postfix increment" operator, similar to the corresponding C
operator.
The operand must be variable whose value is a number. The operator returns
this number, and sets the variable to this number incremented by one. This
means, the operator does not evalute its operand in the standard
way. The postfix operator ++ is used for action and for use of the original
variable value, or only for action (in the latter case, it is equivalent to
prefix ++).
Note: It might be desirable to allow expressions like
a[5]++ or first(l)++, where a is an array and l is a list. This does not work
yet.
(see also: ++x, --x, x--)


x --
Binding power: 111
Example:    x--              ==>  8  (if the value of x was 8 before "x--")
            x                ==>  7  (this is the value of x after "x--", if
                                      this value was 8 before).
This is the "postfix decrement" operator, similar to the corresponding C
operator.
The operand must be variable whose value is a number. The operator returns
this number, and sets the variable to this number decremented by one. This
means, the operator does not evalute its operand in the standard
way. The postfix operator -- is used for action and for use of the original
variable value, or only for action (in the latter case, it is equivalent to
prefix --).
Note: It might be desirable to allow expressions like
a[5]-- or first(l)--, where a is an array and l is a list. This does not work
yet.
(see also: ++x, x++, --x)


x ^ y
Binding powers:  111 110
Examples:           3^4           ==>   81
                    6.25^(-0.5)   ==>   0.4
Scheme function: expt
This is the exponentiation operator. It returns x to the power of y. x should
be positive, unless y is an integer. If complex arithmetic is supported,
virually any numbers are allowed for x and y. The return value has type
integer if x is an integer and y is a nonnegative integer. Note that the
binding powers mean "^" groups right. So 2^3^4 means 2^(3^4) which is
different from (2^3)^4. This complies with mathematical notation.


n!
Binding power: 115
Example:    4!          ==> 24
n! returns the factorial of n. n must be a nonnegative integer.


n div k
Binding powers: 100 100
Examples:            88    div 9    ==>  9
                     (-88) div 9    ==> -9
                     88    div (-9) ==> -9
                     (-88) div (-9) ==>  9
Scheme function: quotient
div returns the result of "integer division", i.e. the integer part of the
quotient. The examples show the behavior of div when one or both operands
are negative. The operands must be integers, k must not be 0.


n mod k
Binding powers: 100 100
Examples:            88    mod 9    ==>  7
                     (-88) mod 9    ==>  2
                     88    mod (-9) ==> -2
                     (-88) mod (-9) ==> -7
Scheme function: modulo
mod returns the result of "modulo" of its operands. If both operands are
positive, this is the usual division remainder.
The examples show the behavior of mod when one or both operands
are negative. The operands must be integers, k must not be 0.


Comparison operators
********************
All comparison operators return a logical value (#t or #f).
Comparison operators may compare numbers for size, or they may compare any
operands for some kind of equality. The operaters that compare numbers for
size are flattable. This means, in particular, you can type in something like
x < y < z
which evaluates to #t (true) if x < y and y < z. However, you cannot mix
different comparison operators. So
4 < 6 <= 9
is parsed as
(4 < 6) <= 9
The subexpression (4 < 6) evaluates to #t (true), it remains to be evaluated
#t <= 9
and this leads to an error, because #t is not a valid operand to <=.
Comparison operators have lower binding powers than arithmetic operators and
higher binding powers than logical operators. So the expression
x < 5+y and z > x*x
is equivalent to
(x < (5+y)) and (z > (x*x)).
There are three comparison operators for equality: =, ==, === . They
correspond, in this order, to the scheme functions equal?, eqv?, eq?. There is
no analogue of the scheme function = (comparison of numbers). For details
of these operators you are referred to the scheme documentation (R5RS, the
Revised (5) Report on the algorithmic language scheme).
The finest equality test is ===, the coarsest one ist =. What is "===" is
also "==", and what is "==" is also "=". Usually, === should be the fastest
test.
Currently there are no operators that compare for unequality. Use something
like
not ( a = b)
instead.

x < y
Binding powers: 80 80
flattable
Example:         4 < 4.01       ==> #t
                 4 < 4.0        ==> #f
                 4 < 5 < 7      ==> #t
                 4 < 5.5 < 5.2  ==> #f
Scheme function:  <
Operator "less than". Compares the operands. The operands should have type
integer or float. The last examples show the effect of flattability (see
begin of this section on comparison operators). Returns a logical value.
(see also: <=, >, >=)


x <= y
Binding powers: 80 80
flattable
Example:         4 <= 4.01         ==> #t
                 4 <= 4.0          ==> #t
                 4 <= 5 <= 7       ==> #t
                 4 <= 5.5 <= 5.2   ==> #f
Scheme function:  <=
Operator "less than or equal to". Compares the operands. The operands should
have type integer or float. The last examples show the effect of flattability
(see begin of this section on comparison operators). Returns a logical value.
(see also: <, >, >=)


x > y
Binding powers: 80 80
flattable
Example:         4 > 3.99       ==> #t
                 4 > 4.0        ==> #f
                 4 > 5 > 1.7    ==> #f
                 4 > 3.5 > 3.2  ==> #t
Scheme function:  >
Operator "greater than". Compares the operands. The operands should have type
integer or float. The last examples show the effect of flattability (see
begin of this section on comparison operators). Returns a logical value.
(see also: <, <=, >=)


x >= y
Binding powers: 80 80
flattable
Example:         4 >= 3.99         ==> #t
                 4 >= 4.0          ==> #t
                 4 >= 5 >= 1.7     ==> #f
                 4 >= 2.5 >= 1.2   ==> #t
Scheme function:  <=
Operator "greater than or equal to". Compares the operands. The operands
should have type integer or float. The last examples show the effect of
flattability (see begin of this section on comparison operators). Returns a
logical value.
(see also: >, <, <=)


a = b
Binding powers:  80 80
Examples:                 5+4     = 9                   ==> #t
                          "abcd"  = "ab"//"cd"          ==> #t
                          {1,2,4} = cons(1,{2,1+3})     ==> #t
                          3       = 3.0                 ==> #f
                          3       = inexact2exact(3.0)  ==> #t
Scheme function: equal?
This is the coarsest equality test. Roughly speaking, what looks equal, is
equal in the sense of "=". See scheme documentation on equal? for more
information.
(see also: ==, ===)


a == b
Binding powers:  80 80
Examples:    5+4      == 9                  ==> #t
             9.0      == 9.0                ==> #t
             3+2.0    == 5.0                ==> #t
             a :: 'ochse,
             b :: a,
             a        == b                  ==> #t
             {1,2,4}  == {1,2,4}            ==> #f
             3        == inexact2exact(3.0) ==> #t
             "ochse"  == "ochse"            ==> #f
Scheme function: eqv?
This equality test is, roughly statet, equivalent to "=" when applied
to number, and equivalent to "===" otherwise (I am, however, not completely
sure).
(See also: =, ===)


a === b
Binding powers:  80 80
Examples:    5+4     === 9                  ==> ?
             9       === 9                  ==> #t (usually)
             9.0     === 9.0                ==> #f (usually)
             'ochse  === 'ochse             ==> #t
             a :: 'ochse,
             b :: a,
             a       === b                  ==> #t
             {1,2,4} === {1,2,4}            ==> #f
             3       === inexact2exact(3.0) ==> ?
             "ochse" === "ochse"            ==> #f
Scheme function: eq?
This is the finest equality test. It returns #t only if the operands point to
the same object (not to equal objects). This means, the target (the addresses
the pointers point to) must be the same. The test 9.0 === 9.0 evaluates to
#f because usually two numbers (numbers constants) are generated and stored
at different places, so the pointer point to different addresses. The test
9 === 9 evaluates (usually) to #t, because small integers are stored in the
places where otherwise pointers are stored. These places are compared by
"===" and hence found to be equal if they contain the same integer. Im am
not sure if these behavior is true for all scheme implementations.
(see also: =, ==).


Logical operators
*****************
There are two logical infix operators, "and" and "or", and one logical
prefix operator, "not". Although there are the two special logical constants
#f (false) and #t (true), all values different from #f are considered true.
This means in particular, the empty list, the number 0 and the empty string
are considered "true".

Logical operators do not necassarily return #t or #f.
The operators "and" and "or" do not evaluate their operands in the standard
way.
Logical operators have lower binding powers than most other operators (lower
than all arithmetic and comparison operators, and lower than some other
operators). Within the logical operators, "not" has the highest binding power,
followed by "and" and "or". I consider this for many situations the most
parentheses saving setting.

a and b
Binding powers:  75 75
flattable
Examples:            3 < 5 and 4 < 7           ==>  #t
                     3 < 4 and 5 < 4           ==>  #f
                     3 < 5 and 8               ==>  8
                     {}    and 4 > 1           ==>  #t
                     1 < 2 and 2 < 3 and "ab"  ==>  "ab"
                     1 < 2 and 2 < 1 and "ab"  ==>  #f
Scheme special syntax: and
This is the logical "and". A chain of operands connected by "and", as in the
last two examples, is transformed to one expression (which, in scheme, would
be written as (and a1 a2 a3 ...)). Thus, a chain of two or more operands,
connected by "and" has to be considered to describe the evaluation. The 
evaluation proceeds as follows: The first operand is evaluated, if its value
is #f, then false is returned, the remaining operands are not evaluated.
If it evaluates to a true value, the second operand is evaluated. If its
value is #f, then #f is returned, and the remaining operands are not
evaluated. The evaluation is continued until a false value is found (in
this case false is returned and the remaining operands are left untouched),
or all operands are evaluated (in that case the value of the last operand
is returned). Because of this evaluation scheme, you may write something like
x=>0 and sqrt(x)
This can be used as long as x has any float value (positive or negative),
because the subexpression sqrt(x) would not be evaluated if x is smaller than
zero.
(see also: or, not)


a or b
Binding powers:  70 70
flattable
Examples:            3 < 5 or 4 < 7            ==>  #t
                     3 < 4 or 5 < 4            ==>  #t
                     3 < 2 or 5 < 4            ==>  #f
                     3 < 5 or 8                ==>  #t
                     8 or 3 < 5                ==>  8
                     {}    or 5 > 7            ==>  {}
                     11 < 2 or 2 < 1 or "ab"   ==>  "ab"
                     11 < 2 or 2 < 3 or "ab"   ==>  #t
Scheme special syntax: or
This is the logical "or". A chain of operands connected by "or", as in the
last two examples, is transformed to one expression (which, in scheme, would
be written as (or a1 a2 a3 ...)). Thus, a chain of two or more operands,
connected by "or" has to be considered to describe the evaluation. The 
evaluation proceeds as follows: The first operand is evaluated, if it has a
true value, this value is returned, the remaining operands are not evaluated.
If it evaluates to #f, the second operand is evaluated. If it has a true
value, this value is returned, and the remaining operands are not evaluated.
The evaluation is continued until a true value is found (in this case this
value is returned and the remaining operands are left untouched), or all
operands are evaluated (in that case the value of the last operand is
returned).
(see also: or, not)


not a
Binding power: 76
Examples:            not 4 < 2     ==> #t
                    not #f        ==> #t
                    not 3<5       ==> #f
Scheme function: not
This is the logical "not" operator. It returns the logical negation of its
operand. It always returns #f or #t. Note that "not" has a relatively small
binding power. This is contrary to some other languages, but more "logical"
(at least, I think so).
(see also: and, or)


Definition, assignment, quoting, functions, evaluation
******************************************************
This is a collection of special operators.

'a
Binding power:  210
Examples:       'ochse   ==>    ochse    [The symbol ochse]
                '(a+b)   ==>    {+,a,b}  [The list of the plus operator and
                                          the symbols a and b. Note that esl
                                          output is normally shown in the
                                          form (+ $a $b), i.e. in scheme
                                          form.]
                '4       ==>    4
                '"kalb"  ==>    "kalb"
Scheme special syntax: '
The quote operator (') returns its unevaluated operand. This means, of course,
non-standard evaluation of its operand. If the operand is an expression,
then the result of parsing the expression to scheme structure will be
returned. Note that the operator ' has high binding power. The prefix operator
quote is semantically exactly equivalent to the ' operator, it has, however,
low binding power.
(See also: quote)


quote a
Binding power:  12
Examples:       quote ochse    ==> ochse
                quote a + b    ==> {+,a,b}  [The list of the plus operator and
                                             the symbols a and b. Note that
                                             esl output is normally shown in
                                             the form (+ $a $b), i.e. in 
                                             scheme form.]
Scheme special syntax: '
The prefix operator "quote" is semantically exactly equivalent to ' (the
usual quote operator). However, it has low binding power.
(See also: ')


a := b
Binding powers:    11 10
Examples:          kalb := 44.3,   ==>    #<unspecified>
                   kalb            ==>    44.3
                   kuh  := 'hund,         #<unspecified>
                   kuh             ==>    hund [the symbol hund]
                   l :: {4,5},            #<unspecified>
                   first(l) := 2,         #<unspecified>
                   l               ==>    {2,5}
                   a :: [2..5,3..4],      #<unspecified>
                   a[3,4] := 7,           #<unspecified>
                   a[3,4]          ==>    7
Scheme special syntax: set!
                       set-car!     (only for the form first(l) := ...)
                       set-cdr!     (only for the form rest(l) := ...)
                       setarrayval! (only for the form a[...]: = ...)
This is the usual assignment operator. The left operand has to be a symbol (or
an array element reference, or an expression of the form first(l) or rest(l)
where l is a variable evaluating to a list). The ":=" operator assigns the
value of the right operand to the left operand. This means, the value of the
right operand becomes the value of the left operand. The assignment ":=" is
used for action. It returns an unspecified value.
The symbol or expression left to the := sign must have a value before (it
must be "defined").
(see also: :, ::)

Caution: Use the form first(l), rest(l) only when l is a variable evaluating
to a list (or a pair). Do not use is for list constants. first({1,2,3}):=4
does not make sense.


a : b
Binding powers:   11 10
Examples:                 a : 4.5,      ==>   4.5
                          a             ==>   4.5
                          c : d : 4+5,  ==>   9
                          c             ==>   9
                          d             ==>   9
Scheme special syntax: set!
Another form of the assignment operator. The difference between ":=" (see
above) and ":" is the fact that ":" returns the value of its right operand.
Moreovere, : groups right, a:b:9 is a:(b:9), both variable are assigned the
number 9. Otherwise stated, multiple assignments of the same value to
different symbols in one infix expression are possible. ":=" should be faster.
":" is used for action and, possibly, for value. It is meant to be used
interactively, for multiple assignments, and in constructs like

while el : pop(l) do ... endwhile;

(See also: :=, ::)

Caution: Do not use something like first(l): ... or a[...] : ...
If you do it anyway, the : operator will work like the := operator (i.e.
an unspecified value will be returned). This is really confusing. There
a warning will be issued in such cases.

                       

a :: b                    [First variant of :: use] 

Binding powers:   11 10
Examples:            a :: 5       ==>    #<unspecified>
                     a            ==>    5
                     b :: 3+1.5   ==>    #<unspecified>
                     b,           ==>    4.5
                     g :: x->x+7  ==>    #<unspecified>
                     g            ==>    [the function that maps x to x+7 ]
Scheme special syntax:  define
This is the define operator. The left operand is a symbol. The right operand
is any valid expression. The right operand is evaluated, the value is assigned
to the just created variable whose name is the left operand. "::" returns the
unspecified value.
Variables must be defined before they can be used. The "::" operator can be
used to assign a (possible unnamed, e.g. created by ->, see below) function
to a variable, as demonstrated in the last to example lines.
Note: The "::" operator is also used for the definition of a function
(procedure). This is described next in this help text. It is also used to
introduce a local variable in the "with" control construct. This is explained
elsewhere (probably in the help text "controls").
(see also: :=, :)

This is the normal use of "::", i.e. it is used to define a variable,
         and to assign a value to this variable. See below for using "::" to
         define a function in a possibly easier way.]


f(x [,y , ...]) :: exp  [Second variant of :: use. 
                         The :: operator used to define a function with
                         less typing effort]
Binding powers: 11 10
Examples:          f(x) :: x*x+7    ==>    #<unspecified>
                   f(4)             ==>    23
                   h(t) :: if t<0 then 0 else sqrt(t) endif 
                                              ==> #<unspecified>
                   h(9)             ==>    3
                   h(-1)            ==>    0
                   g(x,y) :: x*y+y, ==>    #<unspecified>
                   g(4,3)           ==>    15,
                    
This use of :: is only syntactic sugar.
f(x) :: exp
is the same as
f::x-> exp
There may be any number of arguments. So
f()::exp       and f::()->exp are equivalent, as well as
f(x,a):: exp   and f::(x,y)->exp

(See also: ->)


Use of :: in the with construct  [Third use of :: 
                                  It correspondens to scheme's 
                                  (let* ([bindings] ...              ]

This really should be explained elsewhere


x -> a
Binding powers: 15 14
Examples:                 (x->x*x)(5)            ==> 25
                          (x,y)->sin(x)*cos(y)   ==> [an anonymous function]
Scheme: lambda
This is really scheme's lambda. It creates an anonymous function.

(see also: ::)

[*********   Not yet ready ************ ]

     (ampersand        '(prefix eeval esl 256 209))
     (! as prefix)


other

     (doubledot        '(infix doubledotfun esl 89 88))
     (hash             '(infix hashfun esl 89 88))
     (tilde            '(infix tildemac macro 89 88))
     (question         '(prefix show esl 256 200))
     (doubleslash      '(infix string-append scheme 120 120)
     ("in"             '(infix member scheme 80 80))
        ;; "in" has 2 meanings: member predicate (a in list), 
        ;; and in "for a in list do ... ", see procedure "preparekeyconstruct"
