                          plot - overview             revised: 21. Oct 1999 
                          ---------------        last revised: 22. Dec 1999

The procedures in the file "plot.e" can be used to generated function plots.

Plot is, in a sense, similar to gnuplot, but it has fewer options, and it
cannot handle 3-dimensional plots yet. However, it is already useful (at
least for me).

Basically, plot generates an eps (encapsulated postscript) file describing
the plot. A variant of plot (xplot) views the generated eps file using an
eps viewer (e.g. gv or ghostview).

The plot procedures described here are written in the language "esl". The
current implementation of esl is just a preprocessor to scheme. esl code is
translated to scheme code, then the scheme code is executed ("evaluated"). The
translator itself is written in scheme. Compared to scheme, esl has more
standard syntax. There are operators, control constructs and of course
functions. At least for me, esl code is much easier to comprehend. However,
this is a matter of taste. plot could have been written entirely in scheme.
The core of esl should work on any scheme implementation that is R5RS
(Revised 5 Report on the Algorithmic Language Scheme) compliant. Some
of the procedures belonging to "plot" require a system interface (something
like a "system" procedure that can be used to invoke a system command). These
procedure may not work on some scheme implementations.

You have to start scheme and to load the esl translater, if you want to
use plot. Then you have to "load" the plot code. If the system is properly
configurated, all this can be done in one step, e.g. by something like typing

guesl

at the shell prompt. Here guesl is a shell script that starts the scheme
interpreter (in this example, guile is used as scheme system) 
with one or more scheme files that load the esl preprocessor and do
some initializations. The plot code and other esl code will also be loaded.
You will see the esl prompt, or you will get a hint telling you how to get
the esl prompt. Then you can type in esl statements.

You can also use esl in batch mode. When file.e is a text file containing
esl code, you can type at the shell prompt:

guesl file.e

In this case, esl reads the esl code contained in file.e, evaluates the
esl statements, and exits. 
If you type

guesl -p file.e

esl will also process the content of file.e, but after that it will switch
to interactive mode (-p means "prompt"). The file extension .e should be
used for text files containing esl code.

You might type at the esl prompt:

plot(sin,{0,4*pi},{'a4w,'auto});

This will generate an eps file containing a plot of the sine function from
0 to 4 pi. The width of the eps plot is so that it should fit the width of
a4 paper, the height is chosen such that one unit in y direction is equal to
one unit in x direction. The generated eps file is ".tempplot.eps" in the
current directory (you must have write permission for the current directory).
If you have access to an X server and the gv program (viewing postscript and
eps) is installed, you can invoke xplot instead of plot (with the same
arguments). In this case the eps file will be generated in the same way and
then it will be displayed. You can configure another eps viewer (e.g.
ghostview).
The esl expression for the invocation of plot or xplot may extend over several
lines. In interactive mode, the end is usually indicated by a semicolon.

The eps file generated by plot contains only rather simple postscript,
whithout procedure definitions, transformation matrices and other advanced
postscript features.

The necessary coordinate transformations are performed by plot. The unit of
length in the output is always "points" (1/72 of an inch).

You can draw several functions in one plot, you can draw parametric curves
and bar charts. Bar charts, parametric curves and simple curves ("y" as
function of "x") can be mixed in one plot. There are several drawing styles
for curves ("line", "filled", "closed", "impulses"). You can also draw curves
that consist of small symbols (squares, triangles, circles). More symbol types
might be added in the future, or you might add them yourself.

The function range (lowest and highest "x-value", lowest and highest parameter
value, respectively) can be specified for each curve separately, or to be the
same for all curves, or to be common values for some of the curves and special
values for the rest.

Curves can be specified in several ways (for more details, see below):
1. as lambda expression, i.e. something like 
   x -> x*x+sin(x)
   For parametric plots, this has to be of the form
   t -> {t*cos(t),t*sin(t)}
2. as a symbol or an expression that evaluates to a one-argument function,
   e.g. first set
   f(x) :: if x <= 0 then 0 else sin(x) endif,
   then simply use f.
3. As a string of a special kind, illustrated by the following example:
   "f(x) = e^(-x)*sin(x)" 
   The part after the equal sign is an esl expression that should contain the
   variable occuring between the parentheses left to the equal sign.
   This way of specification has the advantage that the string will be used
   automatically for the legend generation.
4. As a two-dimensional array (two columns, at least two rows). The columns
   are used as x-values, y-values, respectively. 

There is a special data format for bar charts (to be explained below).

You can set color, linewidth, dashing style for each curve separately, or you
can use default values.

You can generate a legend. A plot title can be inserted. The plot procedure
tries to be smart to insert coordinate line, coordinate axis and numbers at
the axes automatically.

The plot range (width and height in "points") can be specified, or default
values can be used.

You can add auxiliary graphics elements (e.g. lines, text). The plot procedure
can even be used to insert arbitrary postscipt code in the output file.

Structure of the plot procedures
--------------------------------

There are two procedures for plotting and some auxiliary procedures.

plot(funspecs,funrange,plotrange,extras) is the main plot procedure.

viewplot() displays the last plot (eps file) generated by plot.

xplot(funspecs,funrange,plotrange,extras) combines plot and viewplot.
It generates and shows one plot. Essentially, it calls plot with its own
arguments, and then calls viewplot.

editplot() edits the generated plot file as text. Thus you can modify the
postscript code, if you like. In most cases this should not be necessary.

saveplot([filespec]) copies the last plot file to [filespec].

setmono() sets defaults which are meant to be used for monochrome devices.

setcolored() sets defaults for color devices (this is the default).

Some of these function depend on extended features of the scheme system.
For viewplot, editplot and saveplot to work there must be at least a
rudimentary system interface, i.e. a scheme procedure "system" that takes
a string as argument, and executes this string as an operating system command.
editplot requires an editor (joe is used as standard). As already mentioned
above, viewplot and xplot require the presence of an eps viewer and the
availability of an X system.

To generate a plot file, you should usually use plot (to generate an eps
file) or xplot (to generate an eps file and view it on an X system). In
both cases a temporary file (the name of which is ".tempplot.eps" by default;
you can change this) containing the postscript code is generated in the
current directory. In case of xplot, this file is displayed using an eps
viewer (e.g. ghostview or gv). In one of the first lines of the source file
the default viewer is set (currently to "gv"). You are encouraged to read
and adapt the first part of the source file( plot.e) to your needs.

The arguments to plot and xplot are the same, so only plot is described.

Invoke plot with at least 1 and up to 4 arguments:

plot(funs,funrange,plotrange,extras)

There are default values for all but the first argument. You can omit trailing
arguments if you want to use default values. You cannot, however, omit an
argument if you want to specify a non-default value for an argument right to
that argument (i.e. the position of an argument, if present, must be the
correct one).
You can always use on of the following to specify a default value for an
argument: 0 (the number zero), #f (the logical value "false") or {} (the empty
list).
For example, the following are equivalent: 
plot(sin), plot(sin,0), plot(sin,0,0), plot(sin,0,0,0), plot(sin,#f,{},0).
Also, the following are equivalent (but different to the ones above):
plot(sin,0,{400,200}), plot(sin,{},{400,200}), plot(sin,{},{400,200},#f).

The arguments may be very simple, if your plot task is simple. If you want
to plot the sine function in the intervall from 0 to 10, you may simply use

plot(sin);

If you want to have a different intervall (say - 2 pi to 2 pi), simply specify
the intervall as funrange:

plot(sin,{-2*pi,2*pi});

On the other hand, the arguments may be rather complex.

I tried to make the simple things simple and some not so simple things
possible.

When generating postscript code that draws or fills lines, polygons, circles
and other objects, the coordinates of these objects are usually transformed
to the plot coordinate system (which uses "points" as units of measure). If,
for example, the polygon line for the sine function contains the point
(pi/2,1), these original coordinates are transformed to something like
(123,185). It may be desirable, however, to draw elements whose size are
directly expressed in "points". This should usually be the case for text
sizes, small symbols and similar things.
Plot attributes may contain specifications which depend on a size that should
be expressed in "points" rather than in original coordinates (linewidth,
dashing).
In general, however, the user should have the choice between original
coordinates and plot coordinates. 
This choice is made possible in the plot procedure by selecting one of two
possible data types: strings are interpreted as referring to plot coordinates,
numbers are interpreted as referring to original coordinates. An example:
The width specification {'wi,"0.4"} means the line width is set to 0.4
points, which is a rather thin line. The specification {'wi,0.4} refers to
original coordinates. If you plot the sine function, than this is 40% of
the maximal values of the function, which means it is a very thick line.

The transformation to plot coordinates is always performed by the plot
procedure, not by the postscript processor. It would be possibly to define a
transformation matrix in postscript which is, when in effect, applied to all
plot statements. However, this feature of postscript is not used.

Overview on the meaning of the arguments of plot
------------------------------------------------

The first argument specifies the abstract objects to be plotted (to be called
"plot object" in the sequal). Do not confuse plot objects and "extra" objects;
the latter are specified in the fourth argument. Each plot object may be
accompanied by one or more "plot attributes" that describe features of the
presentation of that object. It is also possible to specify a "range" for
curves as plot attribute. The range is the intervall of the independent
variable for the curve (the "x" in case of simple functions and the parameter
in case of a parametric curve). The range may consist of three numbers
(instead of two numbers). In this case the third number is the "step"
determining the density of function evaluation points. If you specify
{'ra,0,100,0.05} (for details of attribute specification see below), then the
function is plotted for "x" from 0 to 100, it is evaluated at 0, 0.05, 0.1,
0.15, an so on. If the third number is not specified, then a default for the
number of function evaluations is used. Currently the default is set to 100.
To be exact, this means 101 function evaluations (the intervall is divided
into 100 subintervalls).

The second argument specifies the global plot range. This range is only used
for a particular curve if no range for that curve is explicitely definied
in the first argument. Strictly speaking, the second argument is redundant
because a special range for each curve can be specified in the first argument.
However, it may be easier and more natural to specify a commen range for
several functions only once.
The second argument may also contain the "step" (see above).

The third argument specifies the plot range and possibly some features of the
plot (the plot title, the appearance of coordinate lines).
The plot range consist of the width and the height of the plot, expressed in
"points" (1/72 inch). This range does not include the place for the legend,
the title of the plot (if present), and the numbers at the coordinate lines.

The fourth argument is a list of additional graphical elements (lines, text,
...). You can specify virtually anything, if you know how to express it in
the postscript language. Simple things can be expressed without explicit use
of postscript. If you do not want to plot extra elements, omit the fourth
argument.

The transformation of the original coordinates to plot coordinates is usually
automatically determined depending on the function range (in x and in y
direction) and the plot range. Thus, including an additional function in the
first argument may result in another transformation. The extra elements
specified in the fourth argument do not influence the transformation. If you
specify an extra element that is, after transformation, outside the plot
range, it will simply not be drawn (or it will be drawn only partially, if
it is only partially outside).

All plot procedures are procedures in the sense of the esl language (please
note: the name "esl" for this language might change in the future). The
arguments are evaluated in the standard way. This means, in particular, that
you can use expression instead of constant values. Thus you may write {0,2*pi}
instead of something like {0,6.283}.  This is also true when you specify
strings or symbols instead of numbers. Thus, if plot is invoked in the inner
of a loop and you want to generate different titles and filenames, you might
want to do something like

for i from 1 to 3 do
  plot(x->sin(i*x),{0,6*pi},{'auto,{'title,"Example "//number2string(i)}}),
  saveplot("Example_"//number2string(i)//".eps")
endfor;

Lists can be written in the form
list(exp1,exp2, ...)
or equivalently
{exp1, exp2, ...}
If want to pass symbols (in the sense of scheme), you must quote them. For
example, the attribute
{'dr,'circ}
sets the drawing type to the symbol "circle". This means that a sequence of
small circles is plotted to indicate the curve.

There is a special syntax to make the input of short lists in some cases
easier. So you may write 
dr~'sq
instead of {'dr,'sq} or
0..100
instead of {0,100}.
In particular, on german keyboards it is easier to type 0..100 than {0,100}.
However, there are some problems associated with this syntax, so you should
try to understand the standard list syntax first. This standard syntax is used
in the examples for the invocation of plot below.

Two dimensional arrays may be generated for example using the makematrix
function:  makematrix( {{1,3},{2,7},{3,6},{4,5}} ) returns the array
      1   3
      2   7
      3   6
      4   5
(two columns, four rows).
You can also first generate an array with undefined values in the form
a :: [1..4,1..2];
then you can assign values using the syntax a[i,j] := val .

Description of the arguments of plot
------------------------------------

Hint: "Number" always means integer or floating point number. 

plot(funs,funrange,plotrange,extras)

funs (first argument of plot)

      This is a plot object or a nonempty list of plot objects. Each plot
      object is a plot specification or a list whose first element is a plot
      specification and the rest is empty or a list of plot attributes. There
      are several types of attributes, typically, an attribute is a list whose
      first element is a keyword and the rest is a nonempty list of values.

      A plot specification is one of the following:
     
      1. as lambda expression, i.e. something like
         x -> x*x+sin(x)
         In scheme this lambda expression would have been written as
         (lambda (x) (+ (* x x) (sin x)))
         You can use any symbol for the variable (here x), it is always local
         to the lambda expression. There can be any valid esl expression right
         to the symbol -> (arbitrary control constructs, local variables
         introduced with "with a::val1, b::val2, ... do exp1, ... endwith").

         For parametric plots use the form
         t -> {t*cos(t),t*sin(t)}
         i.e. the value for each argument has to be a list of two numbers.

      2. as a symbol or an expression that evaluates to a one-argument
         function. E.g. first define
         f(x) :: if x <= 0 then 0 else sin(x) endif,
         then simply use f as plot specification.

         Hint: This is not really different from the first form of a plot
         specification. In both cases, the plot specification is evaluated
         to a function that should take a real argument an returns a number
         (or returns a list of two numbers, in case of a parametric curve).

      3. as a string of a special kind:
         "f(x) = e^(-x)*sin(x)"
         The part before the equal sign is a symbol followed by a parameter
         enclosed in parentheses, the part after the equal sign is an
         expression (usually) containing the parameter. Of course, you can
         also use an expression returning a string of this kind.
         A string of this kind is processed in the following way. First the
         function symbol is replaced by a synthetically generated symbol (__1,
         __2, ...). Then the equal sign is replaced by two consecutive colons
         (the "define" operator of esl). The result of the above example is
          __1(x) :: e^(-x)*sin(x)
         This is a valid expression defining a procedure __1. This expression
         is evaluated (in the global environment). Then the value of __1 is
         the procedure. The specification is then treated as the symbol __1.
         So far, the result is the same as if
           x -> e^(-x)*sin(x)
         had been specified. There is, however, an additional feature: the
         original string is used for the legend.
         It should be clear from the discussion that the "name" of the
         function within the string does not interfere with other variables
         of the same name. You can even use in one plot invocation several
         strings that start with the same symbol. The list
           {"f(x) = sin(x)","f(x) = cos(x)"} 
         when used as funs argument to plot, will produce the plot of the
         sine and the cosine function.

      4. As a two-dimensional array (two columns, at least two rows). The
         columns are used as x-values and y-values, respectively. Each row
         in the matrix specifies a point. When used in combination with the
         ordinary drawing type (line), a polygon is drawn that connects the
         points in order. When used in combination with a "symbols" drawing
         type, each point is indicated by a symbol of the chosen kind (square,
         triangle, circle). You will obtain a "scatter plot" this way. Scatter
         plots are typically used when there is no order reasonable order of
         the points. Otherwise stated, if the order of the lines in the two
         dimensional array is irrelevant, use symbols as drawing type.

      5. As a list of the form
         {'bars,barwidth,barshift,values}
         This is used for bar charts. The first element identifies the list
         as the data for a bar plot specification. You have to quote the
         symbol bars (unless bars is a variable that evaluates to the symbol
         bars). The fourth list element (values) is an array of two or three
         columns and at least two rows. The values in each row specify one
         bar: if a row contains the number 3 in the first and the number 5 in
         the second column, then a bar of height 5 at the horizontale value 
         (x-value) 3 will be drawn. Thus, one list of the form {'bars, ...}
         specifies a data row (a group of values; usually drawn in the same
         color). barwidth specifies the width of the bar. barshift is a number
         that specifies a shift to the right (barshift > 0) or to the left
         (barshift < 0). If barshift is 0, then no shift will occur. The shift
         is applied to all bars specified in the values argument.
         Shifts are useful if you want to draw groups of bars. You might, for
         example, specify three data specification, all bar charts. All three
         have the same set of x-values, but one specification has zero, one
         has positive and one has negative shift. If the barwidth are the
         same and the shift values are slightly greater then the width, you
         should get reasonably well positioned groups of bars. Do not confuse
         data rows and data groups. See examples 11 and 12 below.
         If the values argument is a matrix with 3 columns, then the third
         value in each row should be a string that will be used as text for
         that bar. The text is placed below the bar. If you have several bar
         data specifications in one plot, then usually only one of them should
         have legend text. The barshift is not applied to the text placement.
         All numbers in a bar chart specification are considered original
         coordinates. This means, you do not have to care about adaption
         to the plotrange. You should, however, be carefull if you want to
         mix bar charts and ordinary curves.      

      Default values for the plot attributes are set in the "plotspecs" global
      variable and may thus be changed. See source file plot.e for the
      definition of plotspecs.

      A plot attribute is one of the following:

      {'dr,t}       t is one of 'line, 'fill, 'clos, 'imp, 'sq, 'circ, 'tri.
                    Default: 'line. 
                    Example: {'dr,'circ}
                    The dr attribute specifies the  drawing type.
                    'line  A line connecting the points (x,f(x)) or
                           (f(t),g(t)) (the latter in the case of parametric
                           curves) is drawn.
                    'fill  The space between the curve and the x-axis is
                           filled. In case of parametric plots, the "sector"
                           between the lines from the coordinate system origin
                           to both ends of the curve and the curve itself is
                           filled.
                    'imp   ("impulses") For each evaluation point x, a line is
                           drawn from (x,0) to (x,f(x)). For paramtric plots,
                           lines are drawn from (0,0) to (f1(t),f2(t)), i.e.
                           to curve points at the evaluation values of the
                           parameter.
                    'clos  combines 'line and 'imp. In case of simple curves,
                           draws also the x-Axis in the funrange.
                    'sq    For each evaluation point x, a symbol (square,
                    'tri   triangle, circle) is drawn at (x,f(x)). The size of
                    'circ  the symbol may be controlled using the size
                           attribute.

      {'rg,r,g,b}   r, g an b are numbers between 0 and 1.
                    Default: depends on the function number, see "plotspecs".
                    Example: {'rg,0.8,0,0.8}
                    Specifies the color for the plot specification. The three
                    numbers are the red, green and blue component of the
                    color, 1 means component is completely present, 0 means
                    not present. The example above specifies a kind of
                    magenta. The 'rg-attribute is transformed to the 
                    postscript procedure setrgbcolor.

      {'wi,w}       w is either a string containing a positive number, or a
                    positive number. 
                    Default: depends on the function number, see "plotspecs".
                    Examples: {'wi,"2"},  {'wi,0.05}, 
                    Specifies the line width for the curve or symbol. It does
                    not influence bar charts. If w is given as number, the
                    width referes to original units, if width is given as
                    string (enclosed in " ... "), then it refers to the plot
                    coordinate system.

      {'da,s1,[s2, s3, ...]}
                   s1 is a positive number or a string containing one or
                   more space separated numbers. If s1 is a string, then
                   only one additional element (s2) is allowed, but not
                   required. It must be a nonnegative number or a string 
                   containing a nonnegative number. s1 may also be an empty
                   string, then there must be no additional element. If s1 is
                   a number, then any sequence of positive numbers may follow,
                   but it is not required to follow. After the numbers,
                   optionally the elements 'o and another nonnegative number
                   may follow.
                   Default: No dashing, i.e. continuous lines. This is 
                            equivalent to ""
                   Examples: {'da,3,5}, {'da,"1 2 3 1",2}, {'da,2,3,'o,2}
                   The 'da attribute sets the dashing. If the second list
                   element is a string and the third element is a number,
                   i.e. something like {'da,"2 3 2",2}, then this is converted
                   to the postscript statement
                   [2 3 2] 2 setdash
                   If the third element is a string, then essentially the same
                   will happen, i.e. {'da,"2 3 2","2"} will result in the same
                   postscript statement.
                   If there is no third list element, then 0 is assumed.
                   If the 'da attribute uses numbers, optionally followed by
                   'o and another number, i.e. something like
                   {'da,0.3,0.1,0.4,'o,0.6}
                   then the numbers are considered to be referring to 
                   original coordinates, and they are scaled to plot 
                   coordinates before composing the postscript dash statement.
                   The number after 'o is written after the closing bracket.
                   If 'o and the following number are missing, then 0 is
                   used. Assuming a scaling factor of 5, the last 'da
                   attribute is transformed to the following postscript code:
                   [ 1.5 0.5 2.0 ] 3.0 setdash
                   Note that in case the element after 'da is a string, the
                   following element, if present, is a number, not a string.
                   For the exact semantics of the generated postscript
                   statement, you are referred to the postscript language
                   reference manual. For short, the numbers between [ ... ] 
                   are the lengths of the consecutive dashes and gaps, the
                   number after ] is the "offset" which determines the point
                   within the dash - gap sequence with which the line begins.
                   You may experiment with the 'da attribute, it is
                   recommended to use plot coordinates (enclosed in " ... ")
                   of order 0.3 to 5. The offset may often be omitted.

      {'si,s}      s is a positive number or a string containing a positive
                   number.
                   Default: "4" (set within "plotspecs", thus may be changed)
                   Examples: {'si,"6"},   {'si,0.4}
                   Determines the size of plot symbols (squares, triangles,
                   circles). If specified as string, plot coordinates are
                   assumed, else original coordinates are assumend. The size
                   thus given (either directly as string or after scaling) is
                   not the exact size in "points", rather it is the "order of
                   magnitude" and chosen such that different symbols of the
                   same size appear to be of approximately equal size.
                   'si does not influence other drawing types.

      {'leg,l}    l is a string.
                  Default: none, i.e. no legend present.
                  Example: {'leg,"The sine function"}
                  This attribute is used to define a legend text for the
                  curve.

      {'ra,l,h,[s]}                     
                 l, h, s are numbers, s is optional, if present, it has to
                 be a positive number.
                 Default: The values specified in the funrange argument. 
                 Example: {'ra,-2,7,0.05}
                 The example means that the function is evaluated and plotted
                 from -2 to 7, the step for function evaluation is 0.05, i.e.
                 the function is evaluated at 9/0.05 + 1 = 181 points. If s is
                 omitted, the default value for the number of subintervalls
                 (currently 100) is used, which means 101 function evaluation.
                 You may change the default in the plot source file (plot.e).
                 The 'ra attribute does not influence the plot specifications
                 "array" and "bar chart".
                  
funrange (second argument of plot)
      funrange has the form
      {low, high, step}
      or
      {low, high}
      funrange specifies the range of the "x"-value (the independent variable)
      or the parameter in case of parametric curves. It does not influence bar
      charts and arrays as plot specifications.
      low, high and (if present) step are numbers. low should be smaller than
      high, step should be positive and much smaller than high-low. If step
      is omitted, then step is chosen such that 100 subintervalls are formed,
      i.e. such that (high-low)/step = 100.
      funrange specifies the range only for those plot objects that have no
      range attribute, and even in this case not for bar charts and arrays.
      The default values for funrange (if nowhere specified) are {0,10}, step
      is set as explained above (this yields step = 0.1 in the default case).
          
plotrange (third argument of plot)
     The standard form of plotrange is a list of the form
     {width,height,p1,...}
     width and height are the width and height of the plot, expressed in 
     "points" (1 point = 1/72 inch = 0.35277778 mm). Additional list elements
     are possible (see below). The plot range specified in this way includes
     the transformed space for all plot specifications (curves, scatter plots,
     bar charts), but not the additional space for coordinate lines, text at
     the lines, the legend (if present), the plot title (if present). For
     these elements the required space is automatically added. Furthermore,
     some "margin" is added. Look at the definitions at the beginning of the
     file "plot.e" for more details. Alltogether this means that the real size
     of the eps plot is greater then the width and height specified in the
     plotrange argument. The final size is written as "BoundingBox" in one
     of the first lines of the generated eps file.
     There are some symbols that can be used instead of explicit width and
     height. If you use the symbol 'a4w instead of width, the plot width is
     chosen such that the whole plot has a width that fits a4 paper width.
     If you choose 'default as width, then some default width (which is
     currently set equal to the a4 width) is selected. If you use the symbol
     'auto as second element (instead of a number that specifies height), the
     height will be chosen such that (maxy - miny)/(maxx-minx) = height/width.
     Here maxx, ... denote the maximal, resp. minimal x and y values obtained
     from the plot objects. The selection 'auto thus means that one unit in
     x direction and one unit in y direction have the same length on the plot,
     or, otherwise stated, circles remain circles, they do not become
     ellipses. This is, however, not always desirable.
     If you do not specify a height, the height will be determined in the
     following way (see below for the values of constants): If 
         minheightwidth <= (maxy-miny)/(maxx-minx) <=maxheightwidth
     then the plot height is determined as if 'auto was specified. Otherwise,
     defaultplotheight is used.
     Currently, the constants mentioned above are set as follows:

     a4plotwidth       = 425   (used for 'a4w)
     defaultplotwidth  = 425   (may be set different from a4plotwidth)
     minheightwidth    = 0.3
     maxheightwidth    = 1.55
     defaultplotheight = 300
    
    If 'auto is the value of the plotrange argument or plotrange is a list
    and 'auto is the first element, then the effect is such as if the list
    {'default,'auto} had been specified (or plotrange would be a list
    beginning with {'default, 'auto}, respectively).
    If the value of plotrange is 0 (zero) or #f (false) or {} (empty list),
    then the effect is the same as with {'default}.

    After width and height (or substitutes, like 'default and 'auto),
    additional plot properties may be set. If you want to set additional
    plot properties, you have to set at least one symbol to describe the
    width and height. So, plotrange may have the value
    {'default,{'title,"Some functions"}}
    but
    {{'title,"Some functions"}}
    is not valid.
    The additional properties are list elements that must follow the width
    and height specification. The additional elements themselves may be in
    any order.

    The following additional plot specification elements are allowed in the
    plotrange argument.

    'nocoords   No coordinate lines are drawn (neither the axes are drawn)
    'noxcoords  No coordinate lines at x values are drawn
    'noycoords  No coordinate lines at y values are drawn

    {'title,titlestring}  Here, titlestring is string (enclosed in " ... ")
                          that is used as plot title. The plot title is
                          placed below the plot.

    Remark: The effect of noxcoords, noycoords does not yet work completely as
    described (09.Oct.1999).

    Hint: Coordinate lines are drawn before the plot objects are drawn. So a
    coordinate line cannot make a plot object invisible. See hints at the end
    of the description of the fourth argument.

extras (fourth argument of plot)
    The argument extras specifies additional graphical elements. It does
    not influence the plot range. You may specify the position and the size
    of the additional elements in original coordinates (if you use numbers) or
    in plot coordinates (using strings that contain numbers).
    If you do not want to add extra graphical elements, set the extras
    argument to 0 or #f or {} (empty list).
    Otherwise, the extras argument is a list of extra elements. Some of the
    extra elements do not draw anything, they only set properties of the
    elements to follow (size, color, ...). Thus, the order of the extra 
    elements may be important.
    The following extra elements are allowed. Whenever applicable, the syntax
    is the same as the syntax of plot attributes in the funs argument of plot.
   
    {'li,x1,y1,x2,y2}  This draws a line from the point (x1,y1) to (x2,y2)
                       (original coordinates).
                       Example: {'li,0,0,2,1}
    {'li,s1,s2}        Here, s1 and s2 are strings that contain two space
                       separated numbers. These pairs of numbers are the
                       starting point and the end point of a line to be drawn,
                       the numbers refer to the plot coordinate system.
                       Example: {'li,"20 50","80 100"}

    {'wi,w}            Sets the linewidth for the following elements. w may be
                       a number or a string, thus referring to original or
                       plot coordinates.
                       Examples: {'wi,0.1}, {'wi,"10"}

    {'rg,r,g,b}        Sets the color for the following elements. The three
                       numbers are the red, green and blue component of the
                       color, 1 means component is completely present, 0 means
                       not present.
                       Example: {'rg,0,0.7,0.7} (a kind of dark cyan)

    {'da,s1,[s2, s3, ...]}
                       Sets the dashing for the following elements. This is
                       the same as setting a dashing for a curve. See above,
                       plot attributes, in the section on the first argument
                       (funs) of plot.

    {'fo,fstring,si}   Sets a font for following text elements. fstring is
                       a string specifying a postscript font (something like
                       "Times-Roman"). si is the size of the font, if you use
                       a number, the size refers to original coordinates, if
                       you use a string containing a number, it refers to
                       plot coordinates. If you want the font size to be
                       measured in the usual units, use a string.
                       Example;  {'fo,"Times-Italic,"24"}

    {'te,text,x,y,[ang]}
                       This specifies a text string to be drawn. text is a
                       string, it is the text. x, y are number, they specify
                       the starting point (in the sense of postscript) of the
                       text in the plot, in original coordinates. The optional
                       ang element sets an angle (the direction) for the text,
                       measured in degrees.
                       Remark: It is not yet possible to specify the starting
                       point in plot coordinates.
                       Examples: {'te,"The text",2,0.5,30}

    {'ps,e1,...}       Copies e1, e2, ... in the postscript file, with some
                       modifications:
                       1. A space is inserted between two elements.
                       2. If an element is found to be an number, then it is
                          checked if there is a following element that is
                          also a number. If this is the case, the two numbers
                          are treated as x and y coordinates referring to the
                          original coordinate system. Therefore, they are
                          transformed to the plot coordinate system, the 
                          transformed coordinates are then written into the
                          postscript file. The Processing of the extra
                          elements proceeds with the element following the
                          two numbers (if there is any).
                       All elements that are strings are copied as they are.
                       Numbers that do not fit the situation above and other
                       elements are written as if they were output by the
                       scheme system.
                       Example: {'ps,"newpath",1,0.3,
                                "moveto currentpoint 20 0 360 arc stroke"}
                       This draws a circle with a radius directed to the right
                       with the radius 20 "points" arround the point (1,0.3),
                       this numbers referring to original coordinates.
                       Of course, you should know some postscript to use this
                       feature.

    Hints:
    1. The rg, wi and da extra elements specify line attributes that are 
       applied to the following drawable extra elements. The same may be true
       for the fo statements, which influences text. These elements remain in
       effect unless new rg, wi, da or fo elements are given. If no such
       arguments are given at all, then certain default values are used. The
       default values are specified in the source file plot.e.
    2. The drawable extra elements are drawn before the plot objects (first
       argument of plot) are drawn. It is expected that this is the usually
       desired behavior (plot objects are the more important ones, they should
       have greater chances to be visible). More completely, the order of
       plotting is the following:
       1. Coordinate lines
       2. Extra objects in the order of there specification
       3. Plot objects in the order of there specification
       Objects that are drawn later may make earlier drawn object invisible
       or partially invisible.
  
*** End of description of arguments of plot ***    

Hints: 
******

Hint 1. In postscript, parentheses (...) are used for string quoting. These
parenthesis are added by the plot procedure automatically when necessary. A
problem may arise when you want to draw text (e.g. as title or legend) that
contains parentheses.
In many situation, there is still no problem, because a postscript string may
contain other postscript strings. Hence properly nested parentheses are ok.
This means that strings that are sent to a postscript system should
either not contain parentheses, or the parentheses should always consist of,
possibly nested, matched pairs of opening and closing parentheses. Because
of this nesting property, arithmetic expressions in the form of strings
usually do not cause any difficulties. If you really want to output unpaired
parentheses or not properly nested parentheses in postscript strings, you have
to mask them with a backslash. You must specify two backslashes, because a
simple backslash will be viewed as a masking character by the scheme system
and not passed literally to the postscript output.
Simple backslashes should be used to mask double quotes.
Example:
{'title,"2\\) The \"second\" plot"}
used as title specification, will produce the following title:
2) The "second" plot


Hint 2. Read the definitions and the comments in the first part of the source
file for the plot procedures (plot.e). This should be comprehensable even if
you do not know esl or scheme. The syntax
name :: value,
or
name :: value;
defines a variable "name" and assigns the value right to :: to this variable.

There are two types of comments in esl. Comments of the first type start with
*** and extend to the end of line. The other comment type is enclosed between
/* ... */, it may contain several lines.
The definitions before the line containing the text "Do not modify ..." in
the file plot.e are meant to be "user modifieable". Some modifications are
easy, some are harder, and some require knowledge of the esl language as well
as the postscript language. The latter is in particular valid for the
definition of the procedures that generate the postscript code for the plot
symbols (see squaresym, circlesym, ...). If you are unsure, make a copy of the
original file before you modify it.


Hint 3. Special syntax for short lists.

You can write lists in the form
c1..c2..c3..
instead of the "official" form
{c1,c2,c3,...}
However, note that ".." is an operator, not a list element delimiter.
So you cannot easily build nested list structures with "..".
c1..(c2..c3) is the same as c1..c2..c3, i.e you do not get nesting.
Usually, you cannot write
c1..{c2..c3}
to obtain the required nesting, because the latter means
{c1,{{c2,c3}}.
However,
{c1,{c2,c3..c4}}
means
{c1,{c2,{c3,c4}}}
which often is what is needed.
You may use "~" instead of "..".
The "~" operator quotes its left operand. This is useful for keywords.
wi~3
means
{'wi,3}
The operator ".." evaluates both perands, "~" evaluates its right operand.

The dots in the operator ".." should not confuse the recognition of numbers.
44...4 means {44,.4}, which is {44,0.4}. You cannot write 4...5 if you mean
{4.,5}. However, 4. ..5 is ok. If unsure, insert spaces.


Note: The ".." and "~" operator might not work in other situations (others
then argument specification for plot). They return a list that contains the
symbol dd_, possible at several positions. The procedure
dropdd (list_exp)
returns the list list_exp (i.e. the argument), but with all occurences of dd_
dropped. Still, some subtile problems might persist.

In any case, do not use ".." and "~" in list that contain more than a few
elements. Do not use them in procedure definition that are meant to be stable.
These operators are meant to be used primarly for interactive input, because
they are easier to type (at least for me, on a german keyboard).

Some examples for the invocation of plot (or xplot)
***************************************************

In all examples, you may plot replace by xplot, if the display of eps files
works. If you use xplot, you will see the effect immediately. Be sure to
specify the correct eps viewer. If this has not been done in an initialization
file, you can use something like

epsviewer := "ghostview";

at the esl prompt.

In the following examples, list are specified in the "correct" way. You may
try the alternative syntax (a..b.., or a~b..c..) as well.

There are 14 examples.

There is an esl source file, called test_plot.e, that contains just these 14
plot invocations and some additional esl code. You may load this file:

loadsys "test_plot";

assuming test_plot.e is in the correct directory (usually /usr/local/esl),
otherwise, you have to specify the full path. test_plot generates 14 
postscript files in the current directory from the following 14 examples.

You may also load another esl source file, test_x_plot.e :

loadsys "test_x_plot";

this assumes the specified eps viewer works, i.e. it is present and has
access to the X server. If this is the case, 14 plots will be displayed. The
14 files are generated and kept as with "test_plot".

The 14 examples
***************

Example 1

plot(sin,{0,4*pi},{'a4w,'auto});

plots the sin function from 0 to 4*pi, using a4 width and height so that
units of length mean the same in both directions.

Example 2

plot(sin,{0,4*pi},'a4w);

essentially the same as before, but height set to standard height.

Example 3

plot(sin,{0,4*pi},{'a4w,{'title,"The sine function"}});

A title will be generated.

Example 4

plot({sin,{'wi,"3"},{'da,"5 2"},{'rg,0,0.6,0.6}},{0,4*pi},{'a4w,'nocoords});

Some plot attributes have been set (width, dashing, color), coordinate lines
are not plotted.

Example 5

plot({{sin,{'dr,'fill}},{cos,{'dr,'imp}},{x->- sin(x),{'dr,'clos}},
      {x->-cos x}},{0,10});

Several functions in one plot. Plot objects that are specified later in the
list may overwrite earlier specified objects. The effect of drawing types
should be clear from this plot. The plot range is not set, so the default
is used. Please note the use of different colors for the objects. These colors
are set in the "plotspecs"-variable.

Example 6 

plot({{"f(x) = sin(x)",{'dr,'fill}},{"f(x) = cos(x)",{'dr,'imp}},
{"f(x) = - sin(x)",{'dr,'clos}},{"f(x) = -cos(x)"}},{0,10});

Here, the functions are specified as strings of the form "f(x) = ...".
The effect is that a legend is generated automatically. Note that the
function symbol (here: f) does not really matter. The part right to the
equal sign must be a valid esl expression.

Example 7

plot({{sin,{'ra,pi/2,3/2*pi}},{cos,{'ra,0,5,0.5}},
{"f(x) = - sin(x)",{'ra,2,8,0.05}},{"f(x) = -cos(x)"}},{0,10});

This should show the effect of different range specifications. Note the effect
of different steps (the curve is not smooth in case of a great step). Also 
note that for the last function the funrange argument of plot ({0,10}) is
used. The legend is created only partially, because different plot
specification types are mixed.

Example 8

plot({ {x->{0.3*x*cos(x),0.3*x*sin(x)}},
       {x->{0.2*x*cos(x),0.4*x*sin(x)},{'dr,'sq}}},
       {0,6*pi,0.05},'auto);

Two parametric plots, one of them using square symbols as drawing type.

Example 9

s :: makematrix({{ 1,4},{3,9},{3.4,5},{2.8,7.1},{1.7,6.3},{4.3,4.9},
{ 1.3,4.5},{2.3,3.9},{3.4,5,05},{4.8,2.1},{3.7,4.3},{2.3,6.9}});
plot({s,{'dr,'tri},{'si,"6"},{'leg,"The scatter variable"}});

This produces a "scatter plot". Note that there are two esl statements,
both of which are terminated by a semicolon. The first defines the array
(matrix) of values, the second invokes plot with the matrix in the first
argument. A legend entry is explicitely generated.

Example 10

s :: makematrix({{ 1,4},{3,9},{3.4,5},{2.8,7.1},{1.7,6.3},{4.3,4.9},
{ 1.3,4.5},{2.3,3.9},{3.4,5,05},{4.8,2.1},{3.7,4.3},{2.3,6.9}});
t :: makematrix({{ 1+pi,4.3},{3.3,7.9},{2.4,5.2},{3.8,7.3},{1.7,5.3},{3.6,4.9},
{ sqrt(2),4.5},{3.3,4.9},{3.6,5,25},{3.8,2.5},{1.7,4.3},{3.3,6.9}});
plot({{s,{'dr,'tri},{'si,"6"},{'leg,"The scatter variable"}},
{t,{'dr,'circ},{'si,"8"}},{x->0.3*x*x}},{2,5});

Two scatter variables and an ordinary function are mixed. Note that
expressions, like sqrt(2), are used in some places in the matrices.

Example 11

b1 :: makematrix({{ 1,4},{3,9},{5,5},{7,7.1},{9,6.3},{11,4.9}});
b2 :: makematrix({{1,4.5},{3,3.9},{5,5.05},{7,2.1},{9,4.3},{11,6.9}});
b3 :: makematrix({{ 1,4.3},{3,7.9},{5,5.2},{7,7.3},{9,5.3},{11,4.9}});
plot({{{'bars,0.3,-0.4,b1},{'wi,"6"},{'leg,"The Red"}},
      {{'bars,0.3,0,b2},{'wi,"6"},{'leg,"The Green"}},
      {{'bars,0.3,0.4,b3},{'wi,"6"},{'leg,"The Blue"}},
      {x->2*sin(x)+4,{'wi,0.2}}},
       {0,12});

This is a more complicated example. It draws bar charts and an ordinary
function. First, 3 matrices are defined in separate esl statements. The plot
statement has a list of 4 plot objects as first argument. The first 3 are
bar chart specifications that use the matrices just defined. The bar widths
are always 0.3, the shifts are -0.4, 0, 0.4, so only the middle chart (the
green one) is placed at the "correct" x values. The legends are specified
explicitely. The widths attributes are only used for the legend (contrary
to curves, there is no influence on the bar charts). The width specification
for the fourth plot object, however, sets the width of the curve. Note that
it is specified in original coordinates. The funrange argument({0,12}) is
only used by the fourth plot object (the sine function).

Example 12

b1 :: makematrix({{ 1,4},{3,9},{5,5},{7,7.1},{9,6.3},{11,4.9}});
b2 :: makematrix(
 {{1,4.5,"One"},{3,3.9,"Three"},{5,5.05,"Fife"},{7,2.1,"Seven"},
                  {9,4.3,"Nine"},{11,6.9,"Eleven"}});
b3 :: makematrix({{ 1,4.3},{3,7.9},{5,5.2},{7,7.3},{9,5.3},{11,4.9}});
plot({{{'bars,0.3,-0.4,b1},{'wi,"6"},{'leg,"The Red"}},
      {{'bars,0.3,0,b2},{'wi,"6"},{'leg,"The Green"}},
      {{'bars,0.3,0.4,b3},{'wi,"6"},{'leg,"The Blue"}},
      {x->2*sin(x)+4,{'wi,0.2}}},
       {0,12},{'default,'noxcoords});

Here, the second array (b2) is a matrix with 3 columns, the third column
contains a text for the bar groups.
Another difference to the foregoing example is the 'noxcoords specification
in the plotrange argument. To make this possible, something describing the
plot width and height has to be set (here: 'default).
Of course, you do not have to redefine the matrices if they do not change
between the plot invocations. Here, you would only have to redefine b2.

Example 13

plot(sin,{0,2*pi},'default,
{{'li,"60 50","200 100"},{'rg,0,0,1},{'li,pi/2,1,3*pi/2,-1},
{'fo,"Times-Bold","36"},{'te,"The blue line",2.2,0.6,-30}});

This contains some extra statements. The first line uses plot coordinates,
the second line uses original coordinates. Note that the color specification
only influences the second line (the order matters). If you view the plot,
you will see that the text "The blue line" is approximatly, but not exactly
parallel to the second line. It is easy to make the text exactly parallel:

First, note that the 'auto feature is in effect. If you do not want to rely on
the default values, specify {'default,'auto} as third argument.
So ratios of y-intervalls to x-intervalls are the same in original and plot
coordinates. Compute the ratio for the second line. It is -2/pi. Take the
arcustangens, you will get an angle, transform this angle to degree. Use the
angle as last element in the 'te statement.
The whole procedure is given in the last example.

Example 14

ang :: atan(-2/pi)*180/pi;
plot(sin,{0,2*pi},{'default,'auto},    
{{'li,"60 50","200 100"},{'rg,0,0,1},{'li,pi/2,1,3*pi/2,-1},
{'fo,"Times-Bold","36"},{'te,"The blue line",2.2,0.6,ang}});

See discussion to example 13.

Please note
***********

Some examples of the invocation of plot are really simple.
So,
xplot(sin)
or
xplot(sin,0..4*pi)
are really simply methods to draw a sine function. You may still simplify
these examples. If the system variable $allowfunasprefix is set to #t, which
is the default value, you may simply type
xplot sin
If you have to plot several function for the same plot intervall, you may
set defaultfunlow and defaultfunhigh to the desired values and then use
a simple plot call like "xplot sin". For example, set
defaultfunlow:=1,defaultfunhigh:=5;
then you may type something like
xplot sin;
xlpot cos;
xplot (sqrt..ln);
xplot {sqrt,ln};
xplot ln;
xplot exp;
or, if f has been defined before,
xplot f;

However, do not consider plot a very high level procedure. plot can be used
as a basis for higher level procedures. These higher level procedures may have
easier interfaces (argument descriptions) then plot itself.
For example, consider the following procedure definition

bar1(+l)::
with
  d :: {},
  k :: 1
do
  if nullq(l) then
    "There must be at least one argument"
  else
    while not nullq(l) do
      d := cons({k,first(l),number2string(k)},d),
      l := rest(l),
      k++
   endwhile,
   xplot({'bars,0.9,0,makematrix(reverse(d))},#f,'auto..'noxcoords)
  endif
endwith;

You may use this procedure to plot bar simple bar charts (with one data row).
Call bar1 in the following way:
bar1(4,3.8,2.7,6,5.7,4.9,4.8)
i.e. for each bar to be plotted use one argument. 
Of course, the bar procedure might be a little more sophisticated. So a
procedure bar2 might be such that a call like
bar2({4,"Jan"},{3.8,"Feb"},{2.7,"Mar"},{6.5,"Apr"},{5.7,"May"},{4.9,"Jun"})
would generate a plot where the numbers are represented by the heights of the
bars and the text strings would be placed under the bars.
