                     esl system features          First version: 06.02.2005
                     *******************

This help topic deals with special features concerning the relation between
esl, scheme and the operating system.

Note that in new esl system (2005-01-29 or later), for the invocation of
procedures with zero arguments the syntax

!proc

may be used alternatively to

proc().

This applies e.g. for ew, parseon, scheme (switching to the scheme prompt).
It does not apply for esl macros.

1. esl variables in scheme
--------------------------

All esl variables are in fact variables in the scheme system - with an
additional Dollar sign before the esl symbol. 
The esl variable 
varname
is the scheme variable
$varname

2. What happens when starting the esl system
--------------------------------------------
Starting the esl system (by guesl, for example) involves the following:
- Starting scheme (guile, for example)
- Loading the file containing the esl code (i.e. the scheme procedures
  that implement the scheme system)
- loading the scheme functions
- processing esl input files and/or starting interactive esl mode.

3. Switching from esl to scheme und back
----------------------------------------
From a running interactive esl, the esl command
scheme();
temporarily suspends esl and starts interactive scheme mode. You can enter
scheme expressions, and you can reference esl variables by using a dollar
sign before the esl variable name (see 1.). You can even create (define)
esl variables this way.
From the scheme prompt, entering
(esl)
brings you back to esl.

Caution: The scheme(); procedure call does not work in an esl script file.

4. load, loadsys, loadesl, loadscheme
----------------------------
The esl procedure invocation
load [filename]
loads esl code from [filename] into the running system, as if the
content of the file was typed in interactively. The loaded files may
themselves contain load statements - file loading may be nested to any
depth. If no full path is specified in the filename, the file is looked for
in the current directory.
The esl procedure loadsys is like load, but looks for the file to be loaded
in the "system" directory which is /usr/local/esl by default.

The scheme procedure invocation
(loadesl [filename] . sys)
loads esl code from the file [filename] to scheme (after the esl system has
been loaded). In scheme, the loaded variables will have the dollar sign.
If the optional parameter sys is specified and has the value 'sys (i.e., it
is the scheme symbol sys), the file is looked for in the "system" directory.

The esl procedure invocation
loadscheme [filename]
loads scheme code. Scheme variables in the file that shall later be accessed
be esl need to start with the dollar sign.

And of course, in scheme you can use
(load [filename])
to load scheme code.

In any case [filename] must evaluate to a string containing the file name.

Note: Use full pathnames when one of these load commands does not work.
[The handling of pathes should be improved].

5. The procedure esl2scheme
--------------------------- 

The esl procedure invocation

esl2scheme [filename]

expects [filename].e to be the name of a text file containing esl code. 
The esl code in [filename].e is "translated" to scheme code, and this
scheme code is written to a newly created file

[filename].scm

This scheme code can subsequently be loaded by

loadscheme [filename].scm

The effect of this load on the running esl is the same as

load [filename]

where [filename] is the original esl file. However, in esl, loadscheme is
faster then load.


6. parseon, parseoff
--------------------

The esl procedure invocation

parseon()

switches on the output of the scheme code that is the result of converting esl
input. There is no further effect. After output of the generated scheme
code, the evaluation takes place as usual.

parseoff()

switches off this feature.

7. Explicit eval
----------------

The esl prefix operator & (ampersand) performs an explicit evaluation.
Usually, you do not need an explicit evaluation, since the esl evaluation
rules (based on the scheme evaluation rules) perform the necessary actions.
However, in some cases an explicit "eval" is useful. In scheme, there is
the eval procedure. The same purpose has the & operator in esl.
Hint: & performs on a guile based scheme the primitive-eval procedure.

Try something like

&'(7*9)

8. Using scheme procedures from esl, schemesymbol
-------------------------------------------------

There are scheme procedures without corresponding esl procedures.
You cannot simply enter the scheme symbol, since a symbol entered in esl
will usually have a starting dollar sign in scheme.
You can access these procedures from esl using the esl procedure schemesymbol.
schemesymbol [string]
(where [string] evaluates to a string) returns the value of the scheme
symbol "[string]". This value might be a scheme procedure. You can now use
this procedure.

An example
Suppose you want to use the scheme procedure string-fill! in esl.

schemesymbol("string-fill!")

evaluates to this procedure (not to the string string-fill!, but to value
of the symbol string-fill!).

You can now apply this procedure, as shown here:

       a::"abcdef",
       (schemesymbol "string-fill!")(a,char "*"),
       a;

The value of the esl symbol a will be "*****".  Note that char "*" returns
the character *. This is different from the string contain the one character
*. string-fill! expects a character as second argument.
In esl, there is currently no direct syntax for character constants.

Caution: 
 - Consider schemesymbol a low level feature. It bypasses, in a sense,
   the esl parser. schemesymbol may be used to destroy a running esl system.
   Do not use it if you can avoid it. The correct way for frequent use of a
   scheme procedure in esl is to create an esl procedure with the same effect.

 - It is not claimed that schemesymbol works for scheme macros.


9. (Continuation of 8.)
In 8., it was shown how to use in esl a scheme procedure (string-fill!)
without corresponding esl procedure.
Another way to achieve the same goal is to define an esl procedure within
scheme. You may proceed as follows, starting from the esl prompt:

scheme();
(define $stringfill string-fill!) 
(esl)
a::"abcdefg";
stringfill(a, char "*");
a;                    ==> *******

The first line switches to scheme. The second line defines the scheme function
$stringfill to be another name for string-fill!. The dollar sign makes the
new symbol available in esl (as stringfill). 
The third line switches back to esl, and the last three lines show how to use
stringfill in esl.
Please note:
  - The dollar sign is essential
  - Use only characters allowed in esl symbols after the dollar sign. Do not
    use - > * or something else. Do use only lower case letters, digits and
    underscore. The first character after the dollar sign should be a lower
    case letter.
  - The newly created procedure will vanish after exiting esl and the
    underlying scheme system. The creation of esl procedures in scheme 
    described here works only in interactive esl.
  - You can write the scheme code that defines an esl function to a text file
    and load the code from esl using loadesl (see 4.)


10. evalstring

evalstring [string]

where [string] should evaluate to a string containing esl code, evaluates
the esl code as if it were typed in. This may even be applied recursively.

Try something like this:

  evalstring "7*8;",
  evalstring "7*9",
  evalstring "a::5,evalstring \"b::a+6\",displayln b";

The end of the string works implicitely like a semicolon.

11. Quoting (quote, ')

Quoting, i.e. using quote or the apostroph, is essentially the same as in
scheme. It prevents the operand from beeing evaluated. Note, however, that
quoting does not stop parsing.
Something like
'*
produces an error in esl, because * is an infix operator, and it does not
stop being so when a quote is placed before it.
The operators ' and quote have exactly the same semantic. Both are prefix
operators. ' has a high and quote a low right binding power.
Try this:

parseon();
'7*8;
quote 7*8;

12. Macros
There is no macro definition feature in esl. There are some "predefined"
macros, though. In a sense, the esl control constructs may be seen as macros.
There are some further macros:
:= ::
(assignment, definition) are (sort of) macros. They do not evaluate both
operands. Also, the functions pop and push are macros, although they are
syntactically treated as functions. pop and push treat lists as stacks.
Since esl version 0.06, help is a macro. This makes it possible to simply
write something like
help functions
instead of help "functions".
