begin
display("This is the module \"test_c_linear_algebra\".\n",
   "Version and date of this test module:"), 
  test_c_linear_algebra_version::"0.01", 
  test_c_linear_algebra_date::"17.08.1999",
  displayln(test_c_linear_algebra_version,"  ",test_c_linear_algebra_date)
end;
displayln("\nScheme system is ",$scheme_system);
displayln("\n",
"Testing procedures in module \"linear_algebra\" using complex arithmetics.\n",
 "It will only work if the scheme system supports complex arithmetics.\n",
   "Version and date of tested module \"linear_algebra\": ",
   linear_algebra_version," ",linear_algebra_date);

/* Generates a system of complex linear equations by multiplying a generated
   matrix  with a generated column vector, then solves the linear systems
   and compares the components of the solution with the components of the
   vector used to generate the solution. The maximal error should be less
   than 1.1e-11. This script can be used for a rudimentary benchmarking:

   Solving the system takes about 9 seconds on a 
    AMD K6 II 350 Mhz, 64MByte RAM, Linux System with esl running on
    top of scm scheme (Version 5c3).
   Same configuration, but guile (version 1.3) instead of scm takes about 
   39 seconds.
   Other scheme systems have not been tested yet. Only systems that support
   complex numbers can be used (elk and rscheme do not work for this reason). 
   If you want more output, remove the comment signs ("***") at the beginning
   of the corresponding lines.
   
*/
with maxerror::0,a::0,b::0,c::0,x::0 do

***displayln("beginning"),
a:[1..100,1..100],
b:zerocol(100),
***displayln("matrices a,b generated"),
for i from 1 to 100 do 
  for j from 1 to 100 do
    a[i,j] := i+i*1i+1/j*1i
  endfor,
  a[i,i] := a[i,i]+10000
endfor,
for i from 1 to 100 do b[i,1] := i*1i+1/(200-i)+i*100*1i endfor,
***displayln("matrices a,b initialized"),
c:matmul(a,b),
***displayln("Matrix a is"),
***matdisplay(a),
***displayln("Column b is"),
***matdisplay(b),
displayln("Column c is now"),
matdisplay(c),
displayln("Solving linear equation"),
x:lineqsolve(a,c),
for i from 1 to 100 do 
  maxerror:max(maxerror,abs(b[i,1]-x[i,1])),
  displayln(b[i,1],x[i,1],b[i,1]-x[i,1]) 
endfor,
displayln("Maximal error of components of the solution is ",maxerror)
endwith;
displayln("\nModule \"test_c_linear_algebra\" (Version ",
  test_c_linear_algebra_version,", ",test_c_linear_algebra_date,")");
